/*
------------------------------------
Controladors
------------------------------------
/**
 * @ngdoc controller
 * @name app.controller:MainCtrl
 * @extends app.rootScope
 * @requires $q
 * @requires app.filter:$filter
 * @requires $state
 * @requires $scope
 * @requires $translate
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires $ionicHistory
 * @requires $timeout
 * @requires app.service:Auth
 * @requires app.service:Usuari
 * @requires app.service:Habitatges
 * @description Controlador principal de l'aplicació
 * @property {Usuari} activeUser Dades de l'usuari actual de l'aplicació, inicialitzat a null
*/
app.controller('MainCtrl', function($q, $filter, $state, $scope, $translate, $ionicLoading,
  $ionicPopup, $ionicHistory, $timeout, Auth, Usuari, Habitatges) {

  $scope.activeUser = null;

  /**
  * @ngdoc function
  * @name toggleLeft
  * @methodOf app.controller:MainCtrl
  * @description Funció per plegar/desplegar el menú lateral
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.toggleLeft = function() {
    $ionicSideMenuDelegate.toggleLeft();
  }; //Fi: toggleLeft

  /**
  * @ngdoc function
  * @name logout
  * @methodOf app.controller:MainCtrl
  * @description Funció per deslogejar un usuari.
  * Fa ús del mètode logout del servei Auth
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.logout = function(){
    //Mostrem un popup de confirmació abans de deslogejar l'usuari
    var logoutPopup = $ionicPopup.confirm({
       title: $filter('translate')('lang_menu_logout'),
       template: $filter('translate')('lang_desconnectar_text'),
       okText: $filter('translate')('lang_ok'),
       cancelText: $filter('translate')('lang_cancelar')
    });
    logoutPopup.then(function(res) {
      // Si accepta, desconectem l'usuari
      if(res){
        //Cridem a la funció logout del servei Auth per destruir
        //la sessió de l'usuair a Firebase i les seves dades a l'aplicació
        Auth.logout();
        $scope.destroyCurrentUser();
        //Funció amb delay per assegurar-nos d'esborrar l'historial de navegació
        $timeout(function () {
          $ionicHistory.clearCache();
          $ionicHistory.clearHistory();
          $ionicHistory.nextViewOptions({ disableBack: true, historyRoot: true });
          $state.go('app.home');
          $ionicLoading.hide();
        }, 5);
      }
    });
  } //Fi: logout

  /**
  * @ngdoc function
  * @name setCurrentUser
  * @methodOf app.controller:MainCtrl
  * @description Funció per desar les dades de l'usuari actual
  * També estableix l'idioma de l'aplicació en funció de l'establert per l'usuari
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.setCurrentUser = function(){
    $scope.activeUser = Usuari.getActiveUser();
    $translate.use($scope.activeUser.data.idioma);
  } // Fi: setCurrentUser


  /**
  * @ngdoc function
  * @name destroyCurrentUser
  * @methodOf app.controller:MainCtrl
  * @description Funció per esborrar les dades de l'usuari actual
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.destroyCurrentUser = function(){
    $scope.activeUser = null;
  } // Fi: destroyCurrentUser

  /**
  * @ngdoc function
  * @name getCurrentUser
  * @methodOf app.controller:MainCtrl
  * @description Funció per obtenir les dades de l'usuari actual
  * @private
  * @return {object} $scope.activeUser Objecte amb les dades de l'usuari actual
  */
  $scope.getCurrentUser = function(){
    return $scope.activeUser;
  }// Fi: getCurrentUser

  /**
  * @ngdoc function
  * @name addFavorits
  * @methodOf app.controller:MainCtrl
  * @description Funció per afegir un habitatge als favorits de l'usuari.
  * Forma part del controllador Main perquè s'utilitza en més d'un controlador.
  * @param {string} key L'identificador de l'habitatge
  * @private
  * @return {external:Promise} Si té éxit, la promesa val true. En cas d'error, la promesa val false
  */
  $scope.addFavorits = function(key){
    var defered = $q.defer();
    if($scope.activeUser){
      //Crida el mètode addFavorits del servei Habitatges de manera asíncrona
      Habitatges.addFavorit(key).then(function(){
        $ionicLoading.show({template:$filter('translate')('lang_add_favorit_ok'), duration:2500});
        defered.resolve(true);
      }, function(error){
        defered.resolve(false);
        $ionicLoading.show({template:$filter('translate')('lang_add_favorit_error'), duration:2500});
      });
    } else {
      //Si l'usuari no està identificat, l'informem que la aqeusta característica
      //és per a usuaris registrats i el redirigim al login
      defered.resolve(false);
      var loginPopup = $ionicPopup.confirm({
         title: $filter('translate')('lang_identificacio'),
         template: $filter('translate')('lang_identificacio_text'),
         okText: $filter('translate')('lang_ok'),
         cancelText: $filter('translate')('lang_cancelar')
      });
      //Si l'usuari es vol identificar, anem al login
      loginPopup.then(function(res) {
        if(res){
          $state.go('app.login');
        }
      });
    }
    return defered.promise;
  } // Fi: addFavorits

  /**
  * @ngdoc function
  * @name removeFavorits
  * @methodOf app.controller:MainCtrl
  * @description Funció per eliminar un habitatge dels favorits de l'usuari.
  * Forma part del controllador Main perquè s'utilitza en més d'un controlador.
  * @param {string} key L'identificador de l'habitatge
  * @private
  * @return {external:Promise} Si té éxit, la promesa val true. En cas d'error, la promesa val false
  */
  $scope.removeFavorits = function(key){
    var defered = $q.defer();
    if($scope.activeUser){
      //Crida el mètode removeFavorits del servei Habitatges de manera asíncrona
      Habitatges.removeFavorit(key).then(function(){
        $ionicLoading.show({template:$filter('translate')('lang_remove_favorit_ok'), duration:2500});
        defered.resolve(true);
      }, function(error){
        defered.resolve(false);
        $ionicLoading.show({template:$filter('translate')('lang_remove_favorit_error'), duration:2500});
      });
    } else {
      //Si l'usuari no està identificat, l'informem que la aqeusta característica
      //és per a usuaris registrats i el redirigim al login
      defered.resolve(false);
      var loginPopup = $ionicPopup.confirm({
         title: $filter('translate')('lang_identificacio'),
         template: $filter('translate')('lang_identificacio_text'),
         okText: $filter('translate')('lang_ok'),
         cancelText: $filter('translate')('lang_cancelar')
      });
      //Si l'usuari es vol identificar, anem al login
      loginPopup.then(function(res) {
        if(res){
          $state.go('app.login');
        }
      });
    }
    return defered.promise;
  } // Fi:removeFavorits

}); // Fi: MainCtrl

/**
 * @ngdoc controller
 * @name app.controller:HomeCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $cordovaGeolocation
 * @requires app.service:Cerca
 * @requires app.service:GoogleMaps
 * @requires app.service:Usuari
 * @description
 * Controlador de la pàgina de benvinguda.Conté el formulari de cerca inicial
 * @property {object} data Objecte que conté les dades de la cerca actual
 * @property {boolean} loading Flag true si s'està carregant dades false si no
*/
app.controller('HomeCtrl', function($scope, $cordovaGeolocation, Cerca, GoogleMaps, Usuari) {

  //Valors inicials de la cerca
  //L'objecte data tindrà la mateixa estructura de dades que cerca
  $scope.data = Cerca.getDadesCerca();
  $scope.loading = false;

  /**
  * @ngdoc function
  * @name cercaHome
  * @methodOf app.controller:HomeCtrl
  * @description Funció per guardar la cerca que fem des del formulari de la home.
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.cercaHome = function(){
    if($scope.data.direccio != ''){
      $scope.data.novaCerca = true; //Forcem la cerca
      //Desem les dades de la cerca a l'Objecte cerca del servei
      Cerca.setDadesCerca($scope.data);
    }
  } //Fi: cercaHome

  /**
  * @ngdoc function
  * @name setTipusCerca
  * @methodOf app.controller:HomeCtrl
  * @description Estableix el tipus de cerca seleccionat per l'usuari com l'actiu
  * @param {string} type El tipus actiu (lloguer o venda)
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.setTipusCerca = function(type) {
    $scope.data.tipusCerca = type;
  }; //Fi: setTipusCerca

  /**
  * @ngdoc function
  * @name getTipusCerca
  * @methodOf app.controller:HomeCtrl
  * @description Retorna true o false si el tipus de cerca
  * passat per paràmetre és l'actual
  * @param {string} type El tipus actiu (lloguer o venda)
  * @return {boolean} True si l'estat és l'actual, false si no ho és
  * @private
  */
  $scope.getTipusCerca = function(type) {
    return type === $scope.data.tipusCerca;
  }; //Fi: getTipusCerca

  //Listener: Observem el valor dels camps data.direccio, data.lat i data.lng per comprovar s i s'estan carregant valors
  $scope.$watchGroup(["data.direccio","data.lat","data.lng"], function(newValue, oldValue) {
    if($scope.data.direccio != ''){
      $scope.loading = false;
    }
  }); //Fi: watchGroup ["data.direccio","data.lat","data.lng"]

});

/**
 * @ngdoc controller
 * @name app.controller:CercaCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $rootScope
 * @requires $state
 * @requires $stateParams
 * @requires $filter
 * @requires $timeout
 * @requires $ionicPopup
 * @requires $ionicLoading
 * @requires $ionicModal
 * @requires $ionDrawerVerticalDelegate
 * @requires $cordovaGeolocation
 * @requires Cerca
 * @requires Habitatges
 * @requires GoogleMaps
 * @requires Firebase
 * @requires URL_FIREBASE
 * @description Controlador de la vista del mapa dels resultats de cerca
 * @property {object} cercleCerca Objecte que representa el radi actual de cerca al mapa
 * @property {object} geoQuery Objecte GeoQuery de Firebase per executar consultes per localització
 * @property {object} cercle Cerca Objecte que representa el radi actual de cerca al mapa
 * @property {boolean} direccioInfo Direcció en la que es desplega el panell d'informació dels habitatges
 * @property {boolean} loadingResultatsCerca Boolean per controlar la càrrega de resultats
 * @property {object} currentSearch Objecte amb les dades de la cerca actual aplicada.
 * @property {object} data Objecte còpia amb les dades de la cerca per aplicar filtres
 * @property {object} habitatgeSeleccionatMapa Objecte que conté les dades de l'habitatge seleccionat al mapa
 * @property {object} habitatgesComparats Array d'habitatges que es mostren a la pantalla de comparació
 * @property {object} activeUser Dades de l'usuari actual
*/
app.controller('CercaCtrl', function($scope, $rootScope, $state, $ionicPopup, $ionicLoading, $ionicModal, $filter, $ionDrawerVerticalDelegate, $timeout, Cerca, Habitatges, GoogleMaps, Firebase, URL_FIREBASE) {
  GoogleMaps.addMap('resultatsCercaMap');
  $scope.cercleCerca;
  $scope.cercleCerca2 = 'a';
  $scope.geoQuery;
  $scope.loadingResultatsCerca = false;
  $scope.currentSearch = Cerca.getDadesCerca(); // Dades de la cerca actual
  //No volem referenciar els dos objectes. Data serà utilitzat com a objecte temporal
  //per guardar els filtres abans d'aplicar-los
  $scope.data = angular.copy($scope.currentSearch );
  $scope.habitatgesComparats = [];
  $scope.habitatgeSeleccionatMapa;
  $scope.activeUser = $scope.getCurrentUser(); //Dades de l'usuari actual

  //Objecte Firebase
  var ref = new Firebase(URL_FIREBASE);


  /**
  * @ngdoc function
  * @name cercaHabitatges
  * @methodOf app.controller:CercaCtrl
  * @description Funció que executa una geoQuery que ens retorna els resultats per
  * la cerca actual. Si els resultats compleixen els filtres de cerca, s'afegeixen
  * als resultats de cerca. S'executa en venir de portada, en aplicar un nou filtre
  * o en cercar per mapa.
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.cercaHabitatges = function() {
    //Ens assegurem que tenim els resultats nets
    Cerca.clearResultats();
    //Netegem el marcadors del mapa
    GoogleMaps.removeAllMarkers('resultatsCercaMap');
    GoogleMaps.resetMarkers('resultatsCercaMap');

    $scope.currentSearch = Cerca.getDadesCerca();
    $scope.data = angular.copy($scope.currentSearch);

    if($scope.currentSearch.lat != 0 && $scope.currentSearch.lng != 0){
      $scope.loadingResultatsCerca = true;
      $timeout($scope.amagarLoading, 8000); //Donem 10 segons a la cerca abans d'amagar el loading
      //Creem un nou objecteGeoFire a la branca a la que volem fer la cerca
      var geoFire = new GeoFire(ref.child('app/tipus-habitatge/' + $scope.currentSearch.tipusCerca + '/localitzacions'));

      //Executem la cerca per localització utilitzant l'objecte geoQuery
      $scope.geoQuery = geoFire.query({
        center: [$scope.currentSearch.lat, $scope.currentSearch.lng],
        radius: $scope.currentSearch.radi
      });// geoQuery

      //Afegim un marcador per la posició de cerca
      var markerPosicio = GoogleMaps.addMarker('resultatsCercaMap', {lat: $scope.currentSearch.lat, lng: $scope.currentSearch.lng}, 'posicio');

      /* Listener: Quan un habitatge deixa de complir les condicions de la cerca*/
      $scope.geoQuery.on("key_exited", function(key, location, distance) {
        Cerca.removeHabitatge(key)
        GoogleMaps.removeMarker('resultatsCercaMap', key);
      }); // geoQuery key_exited

      /* Listener: Quan un habitatge passa a complir les condicions de la cerca*/
      $scope.geoQuery.on("key_entered", function(key, location, distance) {
          Habitatges.getInfoHabitatge(key).then(function(hab){
          hab.key = key;
          //Si l'element compleix els filtres actuals, l'afegim als resultats
          //de la cerca
          if(
          hab.estat == 1
          && ($scope.currentSearch.maxPreu == 0 || hab.preu <= $scope.currentSearch.maxPreu)
          && ($scope.currentSearch.minPreu == 0 || hab.preu >= $scope.currentSearch.minPreu)
          && ($scope.currentSearch.minHab == 0 || hab.numhab >= $scope.currentSearch.minHab)
          && ($scope.currentSearch.maxHab == 0 || hab.numhab <= $scope.currentSearch.maxHab)
          && ($scope.currentSearch.minLav == 0 || hab.numlavabos >= $scope.currentSearch.minLav)
          && ($scope.currentSearch.maxLav == 0 || hab.numlavabos <= $scope.currentSearch.maxLav)
          && ($scope.currentSearch.minSup == 0 || hab.superficie >= $scope.currentSearch.minSup)
          && ($scope.currentSearch.maxSup == 0 || hab.superficie <= $scope.currentSearch.maxHab)
          && ($scope.currentSearch.estat_habitatge == 99 || hab.estat_habitatge == $scope.currentSearch.estat_habitatge)
          && (!$scope.currentSearch.teParquing || hab.te_parquing)
          && (!$scope.currentSearch.teTerrassa || hab.te_terrassa)
          && (!$scope.currentSearch.tePiscina || hab.te_piscina)
          && (!$scope.currentSearch.teCalefaccio || hab.te_calefaccio)
          && (!$scope.currentSearch.teMobles || hab.te_mobles)
          && (!$scope.currentSearch.teAscensor || hab.te_ascensor)
          && (!$scope.currentSearch.teElectrodomestics || hab.te_electrodomestics)
          && (!$scope.currentSearch.teAireCondicionat || hab.te_airecondicionat)
          && (!$scope.currentSearch.tePati || hab.te_pati)
          && (!$scope.currentSearch.teBalcons || hab.te_balcons)
          ) {
            //Ens assegurem de només afegir el resultat un cop
            if($scope.currentSearch.habitatges.indexOf(hab) === -1){
              Cerca.addHabitatge(hab);
              //Creem un nou marcador al mapa per cada habitatge resultant de la cerca
              var newMarker = GoogleMaps.addMarker('resultatsCercaMap', {lat: hab.lat, lng: hab.lng}, hab.key);
              //Afegim un listener per mostrar la informació de l'habitatge en fer clic
              //sobre el marcador.
              newMarker.addListener('click', function() {
                var mapObj = GoogleMaps.getMap('resultatsCercaMap')
                angular.forEach(mapObj.markers,function(element, value){
                  if(element.key != 'posicio'){
                    element.marker.setIcon('./img/ico_casa.png')
                  }
                })
                newMarker.setIcon('./img/ico_casa_active.png');
                $scope.mostrarDetallsAlMapa(hab.key);
              });
            }
          }
        }).then(function(){
          //Amagem la informació de càrrega de resultats
          $scope.amagarLoading();
        }); //getInfoHabitatge
      }); //geoQuery key_entered

      /* Listener: GeoQuery ha acabat la consulta */
      $scope.geoQuery.on("ready", function(key, location, distance) {
      });//geoQuery ready
    }

  } // Fi: cercaHabitatges

  /**
  * @ngdoc function
  * @name amagarLoading
  * @methodOf app.controller:CercaCtrl
  * @description Funció per amagar el loding de la cerca
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.amagarLoading = function(){
    $scope.loadingResultatsCerca = false;
  } //Fi: closeOrdenacio

  /**
  * @ngdoc function
  * @name closeOrdenacio
  * @methodOf app.controller:CercaCtrl
  * @description Funció per tancar la finestra emergent d'ordenació
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.closeOrdenacio = function(){
    $scope.modalOrdenacio.hide();
  } //Fi: closeOrdenacio

  /**
  * @ngdoc function
  * @name closeFiltres
  * @methodOf app.controller:CercaCtrl
  * @description Funció per tancar la finestra emergent de filtres
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.closeFiltres = function(){
    $scope.modalFiltres.hide();
  } //Fi: closeFiltres

  /**
  * @ngdoc function
  * @name clearFiltres
  * @methodOf app.controller:CercaCtrl
  * @description Funció per netejar els filtres (tornem a copiar les dades originals
  * de la cerca a l'objecte temporal data)
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.clearFiltres = function(){
    //Reiniciem els filtres amb la cerca anterior
    $scope.data = angular.copy($scope.currentSearch);
  } //Fi: clearFiltres

  /**
  * @ngdoc function
  * @name filtrarCerca
  * @methodOf app.controller:CercaCtrl
  * @description Apliquem els filtres a la cerca actual
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.filtrarCerca = function(){
    //Assignem les dades del formulari de filtres a la cerca i
    //executem de nou la cerca
    Cerca.setDadesCerca($scope.data);
    $scope.cercaHabitatges();
    $scope.closeFiltres();
  } //Fi: filtrarCerca

  /**
  * @ngdoc function
  * @name ordenarResultats
  * @methodOf app.controller:CercaCtrl
  * @description Funció que ordena els resultats de la cerca en funció del paràmetre passat
  * @param {object} parametres Objecte que conté el camp d'ordenació i l'ordre a seguir (ascendent o descendent)
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.ordenarResultats = function(parametres) {
    Cerca.sortResultats(parametres);
    $scope.modalOrdenacio.hide();
  } // Fi: ordenarResultats

  /**
  * @ngdoc function
  * @name switchComparar
  * @methodOf app.controller:CercaCtrl
  * @description Afegeix o elimina un habitatge de la taula comparativa
  * @param {object} hab Objecte que conté les dades de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.switchComparar = function(hab){

    //Comprovem si l'habitatge estava afegit a la comparació, si hi és l'eliminem
    var index = $scope.habitatgesComparats.indexOf(hab);
    var indexCerca = $scope.currentSearch.habitatges.indexOf(hab);

    if(index > -1){
      $scope.currentSearch.habitatges[indexCerca].esComparat = false; //Per controlar si hem de marcar el checkbox o no
      $scope.habitatgesComparats.splice(index,1);
      $ionicLoading.show({template:$filter('translate')('lang_remove_comparar'), duration:1000});
    } else {
       $scope.currentSearch.habitatges[indexCerca].esComparat = true; //Per controlar si hem de marcar el checkbox o no
       if(hab.imatges){
        hab.primeraImatge = hab.imatges[Object.keys(hab.imatges)[0]]; // A la comparació mostrem la primera imatge
       }
       $scope.habitatgesComparats.push(hab);
       $ionicLoading.show({template:$filter('translate')('lang_add_comparar'), duration:1000});
    }
  } //Fi: switchComparar

  /**
  * @ngdoc function
  * @name removeFromFavorits
  * @methodOf app.controller:CercaCtrl
  * @description Funció per eliminar habitatge resultat de la cerca dels favorits
  *  de l'usuari. Fa ús de la funció removeFavorit del controlador principal
  * @param {string} key Identificador de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.removeFromFavorits = function(key){
    //Obtenim l'índex de l'habitatge dins els resultats de la cerca
    var index = Cerca.getHabitatgeIndexByKey(key);
    if(index !== null){
      $scope.removeFavorits(key).then(function(resposta){
        //Marquem el resultat de la cerca com a no favorit
        $scope.currentSearch.habitatges[index].esFavorit = !resposta;
      });
    } else {
      //Informem a l'usuari que s'ha eliminat l'habitatge dels favorits
      $ionicLoading.show({template:$filter('translate')('lang_remove_favorit_error'), duration:2500});
    }
  } //Fi: removeFromFavorits

  /**
  * @ngdoc function
  * @name addToFavorits
  * @methodOf app.controller:CercaCtrl
  * @description Funció per afegir un habitatge resultat de la cerca als favorits de l'usuari.
  * Fa ús de la funció addFavorits del controlador principal
  * @param {string} key Identificador de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.addToFavorits = function(key){
    //Obtenim l'índex de l'habitatge dins els resultats de la cerca
    var index = Cerca.getHabitatgeIndexByKey(key);
    if(index !== null){
      $scope.addFavorits(key).then(function(resposta){
        //Marquem el resultat de la cerca com a favorit
        $scope.currentSearch.habitatges[index].esFavorit = resposta;
      });
    } else {
      //Informem a l'usuari que s'ha afegit l'habitatge als favorits
      $ionicLoading.show({template:$filter('translate')('lang_add_favorit_error'), duration:2500});
    }
  } //Fi: addToFavorits

  /**
  * @ngdoc function
  * @name mostrarDetallsAlMapa
  * @methodOf app.controller:CercaCtrl
  * @description Funció que desplega un panell amb informació de l'habitatge
  * a la pantalla del mapa.
  * @param {string} key L'identificador de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.mostrarDetallsAlMapa = function(key){
    if(!$scope.habitatgeSeleccionatMapa || $scope.habitatgeSeleccionatMapa.key != key){
      //Obtenim l'índex de l'objecte dins l'array de resultats
      var index = Cerca.getHabitatgeIndexByKey(key);
      $scope.$apply(function(){
        //Assignem les dades de l'habitatge
        $scope.habitatgeSeleccionatMapa = $scope.currentSearch.habitatges[index];
      });
      $ionDrawerVerticalDelegate.openDrawer();
    } else {
      $ionDrawerVerticalDelegate.toggleDrawer();
    }
  } // Fi:mostrarDetallsAlMapas

  //Definició de la finestra emergent de filtres
  $ionicModal.fromTemplateUrl('./templates/filtres.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modalFiltres) {
    $scope.modalFiltres = modalFiltres;
  });

  //Definició de la finestra emergent d'ordenació
  $ionicModal.fromTemplateUrl('./templates/ordenacio.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modalOrdenacio) {
    $scope.modalOrdenacio = modalOrdenacio;
  });

  /* Listener: Observem el valor de data.radi per canviar el zoom del mapa */
  $scope.$watch('data.radi',function(newValue){
    if($scope.data.radi >= 33){
      newZoom = 9;
    } else if($scope.data.radi >= 17){
      newZoom = 10;
    } else if($scope.data.radi >= 9){
      newZoom = 11;
    }else if($scope.data.radi >= 4){
      newZoom = 12;
    } else if($scope.data.radi >= 3){
      newZoom = 13;
    } else {
      newZoom = 14;
    }
    $scope.data.zoom = newZoom;
  }); //Fi: watch data.radi

  //Si venim de portada, iniciem una nova cerca, si no, mostrem l'actual
  if($scope.currentSearch.novaCerca){
    $scope.currentSearch.novaCerca = false;
    $scope.cercaHabitatges();
  }



}); //Fi:CercaCtrl


/**
 * @ngdoc controller
 * @name app.controller:CercaMapaCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $rootScope
 * @requires $stateParams
 * @requires $filter
 * @requires $cordovaGeolocation
 * @requires Cerca
 * @requires GoogleMaps
 * @description Controlador de la vista del mapa dels resultats de cerca
 * @property {boolean} direccioInfo Direcció en la que es desplega el panell d'informació dels habitatges
 * @param {integer} posicio Indiquem si volem cerca per la posició de l'usuari (opcional)
*/
app.controller('CercaMapaCtrl', function($scope, $rootScope, $stateParams, $ionicLoading, $filter, $cordovaGeolocation, Cerca, GoogleMaps) {
  $scope.direccioInfo = true;

  var options = {timeout: 10000, enableHighAccuracy: true};

  //Si hem escollit al menú cerca per mapa, fem una cerca des de la posició de l'usuari
  if($stateParams && $stateParams.posicio == 1){
    $cordovaGeolocation.getCurrentPosition(options).then(function(position){
      //Creem una copia de la cerca model per obtenir un objecte
      //amb la mateixa estructura
      Cerca.netejaFiltres();
      var novaCerca = Cerca.getDadesCerca();
      novaCerca.lat = position.coords.latitude;
      novaCerca.lng = position.coords.longitude;
      Cerca.setDadesCerca(novaCerca);
      $scope.cercaHabitatges()
      angular.element(document).ready(function () {
        $scope.iniciarMapa();
      });
    }, function(error){
      $ionicLoading.show({template:$filter('translate')('lang_error_posicio'), duration:3500});
    });
  } else {
    angular.element(document).ready(function () {
      $scope.iniciarMapa();
    });
  }

  //En tornar de la fitxa no disposem de la geoquery, la tornem a crear
  if(!$scope.geoQuery){
    $scope.cercaHabitatges();
  }

  /**
  * @ngdoc function
  * @name iniciarMapa
  * @methodOf app.controller:CercaMapaCtrl
  * @description Inicia la visualització del mapa utilitzat per mostrar els resultats
  * de la cerca actual. Els marcadors del mapa, els afegreix la funció de cerca
  * del controlador principal al vol.
  * També crea el cercle que indica el radi de cerca i hi afegeix un Listener per
  * controlar-ne el drag.
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.iniciarMapa = function() {

    //Inicialitzem el mapa
    if($scope.currentSearch.lat != 0 && $scope.currentSearch.lng != 0){
      $scope.map = GoogleMaps.iniMap('resultatsCercaMap', $scope.currentSearch.lat, $scope.currentSearch.lng, $scope.currentSearch.zoom, true, true);

      $scope.$parent.cercleCerca = GoogleMaps.addSearchCircle('resultatsCercaMap', $scope.currentSearch.lat, $scope.currentSearch.lng, $scope.currentSearch.radi, '#FFFFFF', '#387EF5');
      //Si movem el cercle de cerca, canvia la consulta;
      //A la funció de cerca (CercaCtrl) s'actualitzaran els resultats de la cerca
      google.maps.event.addListener($scope.cercleCerca, 'drag', function() {

        var newCentre = {lat: $scope.cercleCerca.getCenter().lat(), lng: $scope.$parent.cercleCerca.getCenter().lng()};
        $scope.currentSearch.lat = newCentre.lat;
        $scope.currentSearch.lng = newCentre.lng;
        //Actualitzem també l'objecte temporal per conservar les coordenades en cas d'aplicar filtres
        $scope.data.lat = newCentre.lat;
        $scope.data.lng = newCentre.lng;
        if($scope.geoQuery){
          $scope.geoQuery.updateCriteria({
            center: [newCentre.lat, newCentre.lng]
          });
        }
      });

      GoogleMaps.addAllMarkers('resultatsCercaMap');
      google.maps.event.trigger($scope.map, 'resize');
    }

    //Mirem on són les pestanyes per posicionar el panell d'informació
    if(angular.element(document).find('ion-tabs').hasClass('tabs-top')){
      $scope.direccioInfo = false;
    } else {
      $scope.direccioInfo = true;
    }
  } //Fi:iniciarMapa

  /* Listener: Observem el valor de currentSearch.radi per canviar la consutla geoQuery */
  $scope.$watch('currentSearch.radi', function(newValue){
    if($scope.currentSearch){
      var newRadius = parseInt($scope.currentSearch.radi);
      var newZoom = parseInt($scope.currentSearch.zoom);

      if($scope.cercleCerca && $scope.geoQuery){
        $scope.cercleCerca.setRadius(newRadius * 1000);
        $scope.geoQuery.updateCriteria({
          radius: newValue
        });
      }

      if($scope.map){
        $scope.map.setZoom(newZoom);
        $scope.cercleCerca.setMap($scope.map);
      }
    }
  }); //Fi: watch currentSearch.radi

}); //Fi:CercaMapaCtrl

/**
 * @ngdoc controller
 * @name app.controller:ComparativaCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $rootScope
 * @description Controlador de la vista comparativa entre habitatges
 * @property {string} nextOrdreComparar L'ordre en que s'ordenarà la taula (ascendent o descent)
 * @property {string} currentCampComparar El camp pel que s'ordenarà la taula
*/
app.controller('ComparativaCtrl', function($scope, $rootScope) {
  $scope.nextOrdreComparar = 'asc';
  $scope.currentCampComparar = '';

  /**
  * @ngdoc function
  * @name ordenarComparats
  * @methodOf app.controller:ComparativaCtrl
  * @description Ordena l'array d'habitatges comparats en funció del camp passat
  * per paràmetre. Utilitza la funció d'ordenació de rootScope ordenarArraysObj
  * @param {string} camp Cadena de text amb el camp d'ordenació
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.ordenarComparats = function(camp){
    //Si volem tornar a ordenar pel mateix camp que l'ordenació actual,
    //invertim l'ordre. Si ordenem per un camp nou, primer asc
    if($scope.currentCampComparar == camp){
      $scope.nextOrdreComparar = ($scope.nextOrdreComparar  == 'asc') ? 'desc' : 'asc';
    } else {
      $scope.nextOrdreComparar = 'asc';
    }
    var parametres = {camp: camp, ordre: $scope.nextOrdreComparar}
    $scope.habitatgesComparats.sort($rootScope.ordenarArraysObj(parametres));
    $scope.currentCampComparar = camp;
  } //Fi: ordenarComparats

}); //Fi:ComparativaCtrl

/**
 * @ngdoc controller
 * @name app.controller:FitxaCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $stateParams
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires $filter
 * @requires $cordovaSocialSharing
 * @requires app.service:Habitatges
 * @description Controlador de visualització dels detalls d'un habitatge.
 * @property {Habitatge} habitatge Objecte que conté les dades de l'habitatge visualitzat
 * @property {boolean} loading Boolean que indica si s'estan carregant dades o no
*/
app.controller('FitxaCtrl', function($stateParams, $ionicLoading, $cordovaSocialSharing, $scope, $ionicPopup, $filter, Habitatges){
  $scope.habitatge;
  $scope.loading = false;
  $ionicLoading.show({template:$filter('translate')('lang_load_carregant'), duration:3500});

  var key = $stateParams.key; //Recollim el id passat per paràmetre
  if(key !== null){
    $scope.loadingFitxa = true;
    //Recuperem les dades de l'habitatge
    Habitatges.getInfoHabitatge(key).then(function(hab){
      $scope.habitatge = hab;
      Habitatges.incrementarVisualitzacions(key); //Incrementem el número de visites de l'habitate
      $ionicLoading.hide();
      $scope.loadingFitxa = false;
    }, function(error){
      $ionicLoading.hide();
      $scope.loadingFitxa = false;
      $ionicLoading.show({template:$filter('translate')('lang_fitxa_error'), duration:2500});
    });
  } else {
    $ionicLoading.show({template:$filter('translate')('lang_fitxa_error'), duration:2500});
    $scope.loadingFitxa = false;
  }

  /**
  * @ngdoc function
  * @name removeFromFavorits
  * @methodOf app.controller:FitxaCtrl
  * @description Funció per eliminar l'habitatge dels favorits de l'usuari.
  * Fa ús de la funció removeFavorit del controlador principal
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.removeFromFavorits = function(){
    if($scope.habitatge.key !== null){
      //Eliminem l'habitatge dels favorits i el marquem com a no favorit
      $scope.removeFavorits($scope.habitatge.key).then(function(resposta){
        $scope.habitatge.esFavorit = false;
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_remove_favorit_error'), duration:2500});
    }
  } //Fi:removeFromFavorits

  /**
  * @ngdoc function
  * @name addToFavorits
  * @methodOf app.controller:FitxaCtrl
  * @description Funció per eliminar l'habitatge dels favorits de l'usuari.
  * Fa ús de la funció removeFavorit del controlador principal
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.addToFavorits = function(){
    if($scope.habitatge.key !== null){
      //Afegim l'habitatge als favorits i el marquem com a favorit
      $scope.addFavorits($scope.habitatge.key).then(function(resposta){
        $scope.habitatge.esFavorit = true;
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_add_favorit_error'), duration:2500});
    }
  } //Fi:addToFavorits


  /**
  * @ngdoc function
  * @name whatsappShare
  * @methodOf app.controller:FitxaCtrl
  * @description Funció per compartir a WhatsApp la fixta de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.whatsappShare = function(){
    if(window.plugins && window.plugins.socialsharing){
      window.plugins.socialsharing.shareViaWhatsApp($scope.habitatge.direccio, null , null , function(){
        $ionicLoading.show({template:$filter('translate')('lang_share_ok'), duration:2500});
      }, function(error){
        $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
    }
  } //Fi:whatsappShare


  /**
  * @ngdoc function
  * @name twitterShare
  * @methodOf app.controller:FitxaCtrl
  * @description Funció per compartir a Twitter la fixta de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.twitterShare = function(){
    if(window.plugins && window.plugins.socialsharing){
      window.plugins.socialsharing.shareViaTwitter($scope.habitatge.direccio, null, '', function(){
        $ionicLoading.show({template:$filter('translate')('lang_share_ok'), duration:2500});
      }, function(error){
        $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
    }
  } //Fi:twitterShare

  /**
  * @ngdoc function
  * @name facebookShare
  * @methodOf app.controller:FitxaCtrl
  * @description Funció per compartir a Facebook la fixta de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.facebookShare = function(){
    if(window.plugins && window.plugins.socialsharing){
      window.plugins.socialsharing.shareViaFacebook($scope.habitatge.direccio, null, null, function(){
        $ionicLoading.show({template:$filter('translate')('lang_share_ok'), duration:2500});
      }, function(error){
        $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
    }
  } //Fi:facebookShare

  /**
  * @ngdoc function
  * @name emailShare
  * @methodOf app.controller:FitxaCtrl
  * @description Funció per compartir per email la fixta de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.emailShare = function(){
    if(window.plugins && window.plugins.socialsharing){
      window.plugins.socialsharing.shareViaEmail(
        $scope.habitatge.direccio + ' ' + $scope.habitatge.descripcio, // can contain HTML tags, but support on Android is rather limited:  http://stackoverflow.com/questions/15136480/how-to-send-html-content-with-image-through-android-default-email-client
        'ComAppCasa',
        null, // TO: must be null or an array
        null, // CC: must be null or an array
        null, // BCC: must be null or an array
        null,
        function(){
          $ionicLoading.show({template:$filter('translate')('lang_share_ok'), duration:2500});
        },
        function(error){
          $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
        }
      );
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_share_error'), duration:2500});
    }
  } //Fi:emailShare

  $scope.$on('$ionicView.beforeEnter', function (event, viewData) {
    viewData.enableBack = true;
  });

}); //Fi:FitxaCtrl

/**
 * @ngdoc controller
 * @name app.controller:FitxaMapaCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $stateParams
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires $filter
 * @requires app.service:GoogleMaps
 * @requires app.service:Habitatges
 * @description Controlador de la vista del mapa d'un habitatge.
 * @property {Habitatge} habitatge Objecte que conté les dades de l'habitatge
*/
app.controller('FitxaMapaCtrl', function($stateParams, $ionicLoading, $scope, $ionicPopup, $filter, Habitatges, GoogleMaps){
  $scope.habitatge;
  var key = $stateParams.key;
  $ionicLoading.show({template:$filter('translate')('lang_load_carregant'), duration:3500});
  if(key !== null){
    //Obtenim les dades de l'habitatge
    Habitatges.getInfoHabitatge(key).then(function(hab){
      $scope.habitatge = hab;
      $ionicLoading.hide();
      $scope.iniciarMapa();
    }, function(error){
      $ionicLoading.hide();
      $ionicLoading.show({template:$filter('translate')('lang_fitxa_error'), duration:2500});
    });
  } else {
    $ionicLoading.show({template:$filter('translate')('lang_fitxa_error'), duration:2500});
  }

  /**
  * @ngdoc function
  * @name iniciarMapa
  * @methodOf app.controller:FitxaMapaCtrl
  * @description Funció per inicialitzar la visualització del mapa amb un marcador
  * que indica on es troba l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.iniciarMapa = function() {
    //Inicialitzem el mapa
    GoogleMaps.addMap('fitxaHabitatgeMap');
    GoogleMaps.removeAllMarkers('fitxaHabitatgeMap');
    GoogleMaps.resetMarkers('fitxaHabitatgeMap');
    $scope.map = GoogleMaps.iniMap('fitxaHabitatgeMap', $scope.habitatge.lat, $scope.habitatge.lng, 14, true, true);
    var location = {lat: $scope.habitatge.lat, lng: $scope.habitatge.lng};
    //Afegim el marcador de l'habitatge i centrem el mapa
    GoogleMaps.addMarker('fitxaHabitatgeMap', location, 'posHabitatge');
    GoogleMaps.setCenter('fitxaHabitatgeMap', location);
  } //Fi:iniciarMapa

}); //Fi:FitxaMapaCtrl

/**
 * @ngdoc controller
 * @name app.controller:LlistaAnuncisCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires $filter
 * @requires app.service:Auth
 * @requires app.service:Habitatges
 * @description
 * Controlador del llistat d'anuncis d'un usuari
 * Tabs: Anuncis actius i inactius
 * @property {array} habitatgesUsuari Array d'objectes que contenen la informació
 * dels habitatges de l'usuari
 * @property {string} clase String que ens serveix per posicionar el botó de nou anunci
*/
app.controller('LlistaAnuncisCtrl', function($scope, $ionicLoading, $ionicPopup, $filter, $state, Habitatges, Auth) {

  $scope.clase = '';

  //Movem el botó de nou anunci si les pestanyes son al peu
  if(!angular.element(document).find('ion-tabs').hasClass('tabs-top')){
    $scope.clase = 'sub-footer';
  }

  //Part privada, si no està logejat anem al login
  if(!Auth.isLogged()){
    $state.go('app.login');
  }

  var usuari = $scope.getCurrentUser();
  $scope.habitatgesUsuari = {habilitats: [], deshabilitats: []};
  //Recuperem els habitatges de l'usuari
  if(usuari){
    $ionicLoading.show({template:$filter('translate')('lang_load_carregant'), duration:2500});
    Habitatges.getHabitatgesByUser(usuari.data.id).then(function(habitatgeCol){
      angular.forEach(habitatgeCol, function(value, key) {
        Habitatges.getInfoHabitatge(key).then(function(hab){
          if(hab.imatges){
            hab.primeraImatge = hab.imatges[Object.keys(hab.imatges)[0]];
          }
          if(hab.estat == 1) {
            $scope.habitatgesUsuari.habilitats.push(hab);
          } else if(hab.estat == 2) {
            $scope.habitatgesUsuari.deshabilitats.push(hab);
          }
          $ionicLoading.hide();
        }, function(error){
          $ionicLoading.show({template:$filter('translate')('lang_error_llistat_anuncis'), duration:1000});
        });
      });
    }, function(error){
      $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:2500});
      $state.go('app.login');
    });
  } else {
    $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:2500});
    $state.go('app.login');
  }

  /**
  * @ngdoc function
  * @name deshabilitarAnunci
  * @methodOf app.controller:LlistaAnuncisCtrl
  * @description Funció per marcar un anuni com a deshabilitat, no apareixerà més a les cerques
  * @param {integer} index L'índex dins l'array que ocupa l'habitatge
  * @param {Object} habitatge Objecte que conté les dades de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.deshabilitarAnunci = function(index, habitatge){
    //Mostrem un popup de confirmació
    var deshabilitarPopup = $ionicPopup.confirm({
      title: $filter('translate')('lang_deshabilitar'),
      template: $filter('translate')('lang_deshabilitar_text'),
      okText: $filter('translate')('lang_ok'),
      cancelText: $filter('translate')('lang_cancelar')
    });
    deshabilitarPopup.then(function(res) {
      //Si l'usuari accepta, actualitzem l'estat de l'habitatge i el movem a l'array de deshabilitats
      if(res){
        if(habitatge.key){
          Habitatges.setFieldHabitatge(habitatge.key, 'estat', 2).then(function(){
            //Actualitzem les dades a l'array que mostrem per pantalla
            habitatge.estat = 2;
            $scope.habitatgesUsuari.deshabilitats.push(habitatge);
            console.log($scope.habitatgesUsuari.deshabilitats);
            $scope.habitatgesUsuari.habilitats.splice(index, 1);
            $ionicLoading.show({template:$filter('translate')('lang_ok_deshabilitar_anunci'), duration:2500});
          }, function(error){
            $ionicLoading.show({template:$filter('translate')('lang_error_deshabilitar_anunci'), duration:2500});
          });
        } else {
          $ionicLoading.show({template:$filter('translate')('lang_error_deshabilitar_anunci'), duration:2500});
        }
      }
    });
  } //Fi:deshabilitarAnunci


  /**
  * @ngdoc function
  * @name habilitarAnunci
  * @methodOf app.controller:LlistaAnuncisCtrl
  * @description Funció per marcar un anuni com a habilitat, tornarà a apareixerà a les cerques
  * @param {integer} index L'índex dins l'array que ocupa l'habitatge
  * @param {Object} habitatge Objecte que conté les dades de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.habilitarAnunci = function(index, habitatge){
    //Mostrem un popup de confirmació
    var habilitarPopup = $ionicPopup.confirm({
      title: $filter('translate')('lang_habilitar'),
      template: $filter('translate')('lang_habilitar_text'),
      okText: $filter('translate')('lang_ok'),
      cancelText: $filter('translate')('lang_cancelar')
    });

    habilitarPopup.then(function(res) {
      //Si l'usuari accepta, actualitzem l'estat de l'habitatge i el movem a l'array d'habilitats
      if(res){
        if(habitatge.key){
          Habitatges.setFieldHabitatge(habitatge.key, 'estat', 1).then(function(){
            //Actualitzem les dades a l'array que mostrem per pantalla
            habitatge.estat = 1;
            $scope.habitatgesUsuari.habilitats.push(habitatge);
            $scope.habitatgesUsuari.deshabilitats.splice(index, 1);
            $ionicLoading.show({template:$filter('translate')('lang_ok_habilitar_anunci'), duration:2500});
          }, function(error){
            $ionicLoading.show({template:$filter('translate')('lang_error_habilitar_anunci'), duration:2500});
          });
        } else {
          $ionicLoading.show({template:$filter('translate')('lang_error_habilitar_anunci'), duration:2500});
        }
      }
    });
  } //Fi:habilitarAnunci

}); //Fi: LlistaAnuncisCtrl

/**
 * @ngdoc controller
 * @name app.controller:FavoritsCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires $filter
 * @requires app.service:Auth
 * @requires app.service:Usuari
 * @requires app.service:Habitatges
 * @description
 * Controlador del llistat de favorits d'un usuari
 * @property {array} habitatgesFavoritsUsuari Array d'objectes que contenen la informació dels favorits de l'usuari
*/
app.controller('FavoritsCtrl', function($scope, $ionicLoading, $ionicPopup, $filter, $state, Usuari, Habitatges, Auth) {

  //Part privada, si no està logejat anem al login
  if(!Auth.isLogged()){
    $state.go('app.login');
  }

  var usuari = $scope.getCurrentUser();
  $scope.habitatgesFavoritsUsuari = [];

  //Recuperem els habitatges favorits de l'usuari
  if(usuari){
    $ionicLoading.show({template:$filter('translate')('lang_load_carregant'), duration:2500});
    Usuari.getInfoUsuariById(usuari.data.id).then(function(dadesUsuari){
      angular.forEach(dadesUsuari.favorits, function(value, key) {
        Habitatges.getInfoHabitatge(key).then(function(hab){
          if(hab.imatges){
            hab.primeraImatge = hab.imatges[Object.keys(hab.imatges)[0]];
          }
          if(hab.estat == 1) {
            $scope.habitatgesFavoritsUsuari.push(hab);
          }
          $ionicLoading.hide();
        }, function(error){
          $ionicLoading.show({template:$filter('translate')('lang_error_llistat_favorits'), duration:1000});
        });
      });
    }, function(error){
      $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:2500});
      $state.go('app.login');
    });
  } else {
    $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:2500});
    $state.go('app.login');
  }

  /**
  * @ngdoc function
  * @name removeFromFavorits
  * @methodOf app.controller:FavoritsCtrl
  * @description Funció per eliminar un anunci dels favorits de l'usuari
  * @param {integer} index L'índex dins l'array que ocupa l'habitatge
  * @param {Object} habitatge Objecte que conté les dades de l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.removeFromFavorits = function(index, key){
    if(index !== null){
      $scope.removeFavorits(key).then(function(resposta){
         $scope.habitatgesFavoritsUsuari.splice(index,1);
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_remove_favorit_error'), duration:2500});
    }
  } //Fi:removeFromFavorits

}); //Fi:FavoritsCtrl

/**
 * @ngdoc controller
 * @name app.controller:FormAnunciCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $stateParams
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires $filter
 * @requires app.service:Auth
 * @requires app.service:Usuari
 * @requires app.service:Habitatges
 * @description
 * Controlador del formulari de creació d'un nou anunci
 * @property {object} posicioUsuari Objecte amb les coordenades amb la posicio de l'usuari
 * @property {array} Fotos Array de fotografies de l'anunci
 * @property {object} map Objecte que conté el mapa de l'anunci
 * @property {boolean} newFormAnuncis True si és un nou anunci, false si n'estem editant un
 * @property {string} titolFormulariAnuncis Títol de la pantalla en funció si és un nou anunci o n'estem editant un
 * @property {Habitatge} data Objecte que conté les dades de l'anunci
 * @params {string} - key Identificador de l'habitatge a modificar (opcional)
*/
app.controller('FormAnunciCtrl', function($stateParams, $scope, $state, $ionicPopup, $cordovaGeolocation, $cordovaCamera, $cordovaImagePicker, $ionicLoading, $filter, Usuari, Habitatges, GoogleMaps, Auth) {

  if(!Auth.isLogged()){
    $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:2500});
    $state.go('app.login');
  }

  var usuari = $scope.getCurrentUser();
  var options = {timeout: 10000, enableHighAccuracy: true};

  $scope.posicioUsuari = {};
  $scope.fotos = [];
  $scope.map;
  //Si per paràmetre rebem un identificador, formulari de modificació. N'obtenim les dades
  if($stateParams.key){
    var key = $stateParams.key
    $scope.newFormAnuncis = false;
    $scope.titolFormulariAnuncis = $filter('translate')('lang_modi_anunci');
    $ionicLoading.show({template:$filter('translate')('lang_load_carregant'), duration:2500});
    Habitatges.getInfoHabitatge(key).then(function(dadesHabitatge){
      if(dadesHabitatge){
        $scope.data = angular.copy(dadesHabitatge);
        $scope.data.key = $stateParams.key;
        angular.forEach(dadesHabitatge.imatges, function(value, key){
          $scope.fotos.push({id: key, data64: value});
        });
      } else {
        $ionicLoading.show({template:$filter('translate')('lang_error_no_habitatge_data'), duration:2500});
        $state.go('app.anuncis');
      }
    }, function(error){
      $ionicLoading.show({template:$filter('translate')('lang_error_no_habitatge_data'), duration:2500});
      $state.go('app.anuncis');
    });
  } else {
    //Si no rebem identificador, nou habitatge
    $scope.newFormAnuncis = true;
    $scope.titolFormulariAnuncis = $filter('translate')('lang_nou_anunci');
    $scope.data = {
      direccio: '',
      poblacio: '',
      estat: 1, // 1: Actiu | 0: Inactiu
      estat_habitatge: 0, //  1: Nou, 2: Seminou, 3: Reformat, 4: Per reformar, 5: Ruïna, 6 Segona mà
      tipus: 'venta', //0: Venta | 1: Lloguer
      carrer: '',
      cp: '',
      numero: '',
      pis: '',
      porta: '',
      pais: '',
      preu: '',
      superficie: '',
      numhab: '',
      numlavabos: '',
      te_parquing: false,
      te_terrassa: false,
      te_piscina: false,
      te_calefaccio: false,
      te_mobles: false,
      te_ascensor: false,
      te_electrodomestics: false,
      te_airecondicionat: false,
      te_pati: false,
      te_balcons: false,
      descripcio: '',
      lat: 0,
      lng: 0,
      id_usuari: null,
      num_favorits:0,
      num_visualitzacions:0,
    };
  }

  /**
  * @ngdoc function
  * @name iniciarMapa
  * @methodOf app.controller:FormAnunciCtrl
  * @description Funció per inicialitzar el mapa del l'habitatge
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.iniciarMapa = function() {
    //Inicialitzem el mapa
    GoogleMaps.addMap('nouAnunciMap');
    GoogleMaps.resetMarkers();
    $cordovaGeolocation.getCurrentPosition(options).then(function(position){
      $scope.posicioUsuari.lat = position.coords.latitude;
      $scope.posicioUsuari.lng = position.coords.longitude;
      $scope.map = GoogleMaps.iniMap('nouAnunciMap',$scope.posicioUsuari.lat, $scope.posicioUsuari.lng, 14, false, false);
    },  function(error){
      //Si no podem iniciar el mapa a la posició de l'usuari, el creem igualment
      $scope.map = GoogleMaps.iniMap('nouAnunciMap', 0, 0, 14, false, false);
    });
  } //Fi:iniciarMapa

  /**
  * @ngdoc function
  * @name removeFoto
  * @methodOf app.controller:FormAnunciCtrl
  * @description Eliminem una foto carregada previament a l'array de fotos
  * @param {object} event L'esdeveniment que ha invocat la funció
  * @index {integer} L'identificador de la foto dins l'array de fotos
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.removeFoto = function(event, index){
    event.preventDefault();
    if($scope.fotos && $scope.fotos[index]){
      $scope.fotos[index].data64 = null;
    }
  } //Fi:removeFoto

  /**
  * @ngdoc function
  * @name addFoto
  * @methodOf app.controller:FormAnunciCtrl
  * @description Afegim una foto a l'array de fotos (no es puja fins a enviar l'anunci)
  * @param {object} event L'esdeveniment que ha invocat la funció
  * @private
  * @returns {undefined} No retorna valors
  */
  /* Alerta: Aquesta funcionalitat només funciona en simuladors i dispositius reals, no en navvegadors*/
  $scope.addFoto = function(event) {
    event.preventDefault();
      var alertPopup = $ionicPopup.show({
        title: $filter('translate')('lang_origen_foto'),
        template: $filter('translate')('lang_origen_foto_text'),
        buttons: [
          {
            text: $filter('translate')('lang_camera'),
            type: 'button-positive',
            onTap: function(e) {
              return 1;
            }
          },
          {
            text: $filter('translate')('lang_album'),
            type: 'button-positive',
            onTap: function(e) {
              return 2;
            }
          },
        ]
      });

      //Un cop seleccionat l'origen de la foto, mostrem el menú corresponent
      alertPopup.then(function(origen) {

        if(origen){
          try{
            var options = {
                quality : 75,
                destinationType : Camera.DestinationType.DATA_URL,
                sourceType : origen === 1 ? Camera.PictureSourceType.CAMERA : Camera.PictureSourceType.PHOTOLIBRARY,
                allowEdit : true,
                encodingType: Camera.EncodingType.JPEG,
                popoverOptions: CameraPopoverOptions,
                targetWidth: 500,
                targetHeight: 500,
                saveToPhotoAlbum: false
            };

            $cordovaCamera.getPicture(options).then(function(imageData) {
                //Generem un id únic a partir del timestamp actual per evitar conflictes
                $scope.fotos.push({id: 'foto_' + new Date().getTime(), data64: imageData});
                $ionicLoading.show({template: $filter('translate')('lang_ok_fotos_carrega'), duration:2500});
            }, function(error) {
                $ionicLoading.show({template:$filter('translate')('lang_error_fotos_carrega'), duration:3500});
            });
          } catch(e){
            $ionicLoading.show({template:$filter('translate')('lang_error_no_camera'), duration:3500});
          }
        }
      });
  }// Fi:addPhoto

  /**
  * @ngdoc function
  * @name UpdateAnunci
  * @methodOf app.controller:FormAnunciCtrl
  * @description Actualitzem les dades de l'anunci modificat a la base de dades
  * @param {object} event L'esdeveniment que ha invocat la funció
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.UpdateAnunci = function(event){
    event.preventDefault();
    $ionicLoading.show({template:$filter('translate')('lang_load_actualitzant')});
     //Ens assegurem que l'usuari està logejat
    if(Auth.isLogged()){
      var dadesHabitatge = angular.copy($scope.data);
      dadesHabitatge.modification = new Date().getTime();
      dadesHabitatge.dadesUsuari = null; //Eliminem les dades de l'usuari que no hem  de guardar
      Habitatges.updateHabitatge(dadesHabitatge, $scope.fotos).then(function(resposta){
        var alertPopup = $ionicPopup.alert({
           title: $filter('translate')('lang_nou_anunci_modi_ok_title'),
           template: $filter('translate')('lang_nou_anunci_modi_ok_title'),
           okText: $filter('translate')('lang_ok'),
        });
        //Un cop modificat l'anunci, anem a la pantalla
        //de llistat d'anuncis
        alertPopup.then(function(res) {
          $state.go('app.anuncis');
        });
      }, function(error){
          $ionicLoading.show({template:$filter('translate')('lang_error_update_anunci'), duration:3500});
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:3500});
      $state.go('app.login');
    }
    $ionicLoading.hide();
  } //Fi:UpdateAnunci

  /**
  * @ngdoc function
  * @name AddAnunci
  * @methodOf app.controller:FormAnunciCtrl
  * @description Creem un nou anunci a la base de dades
  * @param {object} event L'esdeveniment que ha invocat la funció
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.AddAnunci = function(event) {
    event.preventDefault();
    $ionicLoading.show({template:$filter('translate')('lang_load_creant')});

    //Ens assegurem que l'usuari està logejat
    if(Auth.isLogged()){
      //Creem un objecte amb totes les dades de l'anunci
      var nouHabitatge = angular.copy($scope.data);

      //Convertim alguns dels paràmetres per assegurar-nos que tenen un valor correcte
      nouHabitatge.id_usuari = usuari.data.id,
      nouHabitatge.preu = parseInt($scope.data.preu),
      nouHabitatge.superficie = parseInt($scope.data.superficie),
      nouHabitatge.numhab = parseInt($scope.data.numhab),
      nouHabitatge.numlavabos = parseInt($scope.data.numlavabos),
      nouHabitatge.estat_habitatge = parseInt($scope.data.estat_habitatge),
      nouHabitatge.creation = new Date().getTime(),
      nouHabitatge.modification = new Date().getTime(),

      Habitatges.createHabitatge(nouHabitatge, $scope.fotos).then(function(resposta){
        var alertPopup = $ionicPopup.alert({
           title: $filter('translate')('lang_nou_anunci'),
           template: $filter('translate')('lang_nou_anunci_ok_titol'),
           okText: $filter('translate')('lang_ok')
        });
        //Un cop afegit l'anunci, anem a la pantalla
        //de llistat d'anuncis
        alertPopup.then(function(res) {
          $state.go('app.anuncis');
        });
        $state.go('app.anuncis');
      }, function(error){
        $ionicLoading.show({template:$filter('translate')('lang_error_new_anunci'), duration:3500});
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_error_noident'), duration:3500});
      $state.go('app.login');
    }
    $ionicLoading.hide();
  } //Fi:AddAnunci

  //Listener: Observem els valors de data.direccio, data.lng i data.lat per afegir un marcador al mapa a la posició del nou habitatge
  $scope.$watchGroup(["data.direccio","data.lng","data.lat"], function() {
    if($scope.data && $scope.data.lat){
      GoogleMaps.removeAllMarkers('nouAnunciMap');
      var location = {lat: $scope.data.lat, lng: $scope.data.lng};
      GoogleMaps.addMarker('nouAnunciMap', location, 'posNewAnunci');
      GoogleMaps.setCenter('nouAnunciMap', location);
    }
  });

}); //Fi:FormAnunciCtrl

/**
 * @ngdoc controller
 * @name app.controller:LoginCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $rootScope
 * @requires $state
 * @requires $filter
 * @requires $translate
 * @requires $ionicLoading
 * @description Controlador del formulari de creació d'un nou anunci
 * @property {object} firebaseError Objecte que conté l'error que retorna Firebase
 * @property {boolean} loading True si s'estan carregant dades false si no
 * @property {object} data Obecte que conté les dades de l'usuari
*/
app.controller('LoginCtrl', function($filter, $state, $scope, $rootScope, $ionicLoading, Usuari, Auth) {

  $scope.firebaseError = null;
  $scope.loading = false;

  $scope.data = {
    username: '',
    password : '',
    error: ''
  };

  /**
  * @ngdoc function
  * @name Login
  * @methodOf app.controller:LoginCtrl
  * @description Funció que s'encarrega de logejar l'usuari i recuperar-ne les dades
  * @param {event} event Esdeveniment que crida a l'acció de crear un nou compte
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.Login = function(event) {
    event.preventDefault();

    //Mostrem l'animació de càrrega
    $ionicLoading.show({template:$filter('translate')('lang_load_accedint')});
    $scope.loading = true;
    //Comprovem que disposem de totes les dades
    if($scope.data.username != '' && $scope.data.password != ''){
      var credencials = {};
      credencials.email = $scope.data.username;
      credencials.password = $scope.data.password;

      Auth.login(credencials).then(function(authData){
        //Obtenim les dades adicionals de l'usuari
        Usuari.setInfoActiveUsuari(authData.uid, authData.password.email).then(function(){
          $scope.setCurrentUser();
          $scope.loading = false;
        }, function(error){
          $scope.data.error = error; //Capturem l'error però avançem
          $scope.loading = false;
          $ionicLoading.show({template:$filter('translate')('lang_error_dades'), duration:3500});
        });

        //Si el destí original era una pàgina privada, hi anem
        //si no, anem al login
        if($rootScope.postLogInState){
          $state.go($rootScope.postLogInState);
          $rootScope.postLogInState = null;
        } else {
          $state.go('app.home');
        }
      }, function(error){
        $scope.loading = false;
        //Capturem l'error que retorna Firebase
        $scope.data.error = error.code;
        switch (error.code) {
          case 'INVALID_USER':
          case 'INVALID_PASSWORD':
              $scope.data.errorFirebase = 'lang_firebase_error_' + error.code;
              break;
          default:
              $scope.data.errorFirebase = 'lang_firebase_error_ERROR_GENERIC';
        }
      });
    } else {
      $scope.data.errorFirebase = 'lang_firebase_error_ERROR_GENERIC';
    }
    //Amagem l'animació de càrrega
    $ionicLoading.hide();
  } //Fi:Login

}); //Fi:LoginCtrl

/**
 * @ngdoc controller
 * @name app.controller:SignUpCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $filter
 * @requires $translate
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires app.service:Usuari
 * @description Controlador del formulari de creació d'un nou compte d'usuari
 * @property {object} data Objete que conté les dades de l'usuari a crear
*/
app.controller('SignUpCtrl', function($scope, $ionicPopup, $ionicLoading, $translate, $state, $filter, Usuari) {

  $scope.data = {
    username: '',
    password: '',
    password2: '',
    telefon: '',
    nom: '',
  };

  /**
  * @ngdoc function
  * @name SignUp
  * @methodOf app.controller:SignUpCtrl
  * @description Funció de creació d'un nou usuari a l'aplicació
  * @param {event} event Esdeveniment que crida a l'acció de crear un nou compte
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.SignUp = function(event) {
    event.preventDefault();

    //Mostrem l'animació de càrrega
    $ionicLoading.show({template:$filter('translate')('lang_load_creant')});

    //Comprovem que disposem de totes les dades obligatòries
    if($scope.data.username != ''
      && $scope.data.nom != ''
      && $scope.data.password != ''
      && $scope.data.password == $scope.data.password2){

      var credencials = {};

      credencials.username = $scope.data.username;
      credencials.nom = $scope.data.nom;
      credencials.password = $scope.data.password;
      credencials.telefon = $scope.data.telefon;
      credencials.nivell = 2;
      credencials.showEmail = false;
      credencials.showPhone = false;
      credencials.idioma = $translate.use();

      //Si hem seleccionat el checkbox de venedor, canviem el perfil
      if($scope.data.vendre){
        credencials.nivell = 3;
        credencials.showEmail = true;
        credencials.showPhone = true;
      }
      //Cridem al mètode createUser de l'Usuari passant com a paràmetre les credencials de l'usuari
      Usuari.createUser(credencials).then(function(authData){
        credencials.id = authData.uid;
        credencials.email = authData.email;
        //Si hem pogut crear l'usuari, establim les dades adicionals
        Usuari.setInfoUsuari(credencials);
        var alertPopup = $ionicPopup.alert({
           title: $filter('translate')('lang_nou_compte_ok_titol'),
           template: $filter('translate')('lang_nou_compte_ok_text'),
           okText: $filter('translate')('lang_ok')
         });

        //Un cop registrat, anem a la pantalla de benvinguda
        alertPopup.then(function(res) {
          $state.go('app.home');
        });
      } , function(error){
        //Capturem l'error que retorna Firebase i el mostrem
        $scope.data.error = error.code;

        switch (error.code) {
          case 'EMAIL_TAKEN':
          case 'INVALID_EMAIL':
              $scope.data.errorFirebase = 'lang_firebase_error_' + error.code;
              break;
          default:
              $scope.data.errorFirebase = 'lang_firebase_error_ERROR_NEWUSER_GENERIC';
        }
      });
    } else {
      $scope.data.errorFirebase = 'lang_firebase_error_ERROR_GENERIC';
    }

    //Amagem l'animació de càrrega
    $ionicLoading.hide();
  } //Fi:SignUp

}); //Fi:SignUpCtrl


/**
 * @ngdoc controller
 * @name app.controller:LostPasswordCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $filter
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires app.service:Usuari
 * @description Controlador del formulari de recuperació de contrasenya
 * @property {object} data Objete que conté el correu de l'usuari
*/
app.controller('LostPasswordCtrl', function($scope, $ionicPopup, $ionicLoading, $state, $filter, Usuari ) {

  $scope.data = {
    email: '',
  };

  /**
  * @ngdoc function
  * @name LostPassword
  * @methodOf app.controller:LostPasswordCtrl
  * @description Funció de recuperació de la contrasenya d'un usuari
  * @param {event} event Esdeveniment que crida a l'acció de recuperar la contrasenya
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.LostPassword = function(event) {
    event.preventDefault();

    //Mostrem l'animació de càrrega
    $ionicLoading.show({template:$filter('translate')('lang_load_creant')});

    //Comprovem que disposem de totes les dades
    if($scope.data.email != ''){
      //Cridem al mètode corresponent d'usuari de recuperació de contrasenya
      Usuari.resetPassword($scope.data.email).then(function(resposta){
        //Mostrem un missatge de confirmació
        var alertPopup = $ionicPopup.alert({
          title: $filter('translate')('lang_recuperacio_contrasenya'),
          template: $filter('translate')('lang_recuperacio_contrasenya_text'),
          okText: $filter('translate')('lang_ok')
        });

        //Un cop enviada la contrasenya, anem a la pantalla de login
        alertPopup.then(function(res) {
          $state.go('app.login');
        });
      } , function(error){
        //Capturem l'error que retorna Firebase i el mostrem
        $scope.data.error = error.code;

        switch (error.code) {
          case 'INVALID_USER':
              $scope.data.errorFirebase = 'lang_firebase_error_' + error.code;
              break;
          default:
              $scope.data.errorFirebase = 'lang_firebase_error_ERROR_LOSTPASSWORD_GENERIC';
        }
      });
    } else {
      $scope.data.errorFirebase = 'lang_firebase_error_ERROR_GENERIC';
    }

    //Amagem l'animació de càrrega
    $ionicLoading.hide();
  } //Fi:LostPassword

}); //Fi:LostPasswordCtrl

/**
 * @ngdoc controller
 * @name app.controller:ConfigCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $filter
 * @requires $translate
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires app.service:Usuari
 * @description
 * Controlador del formulari de recuperació de contrasenya
 * @property {object} configuracio Objecte que conté les dades de la configuració
*/
app.controller('ConfigCtrl', function($translate, $scope, $ionicPopup, $ionicLoading, $state, $filter, Usuari) {

  var activeUser = $scope.getCurrentUser();
  $scope.configuracio = {
    idioma: "" + $translate.use() //Forcem la conversió a string
  };

  /**
  * @ngdoc function
  * @name canviarIdioma
  * @methodOf app.controller:ConfigCtrl
  * @description Funció per canviar l'idioma de l'aplicació i de l'usuari
  * @param {string} idioma L'idioma en que volem veure l'aplicació
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.canviarIdioma = function(idioma){
    $translate.use(idioma);
    if(activeUser && (idioma != activeUser.data.idioma)){
      Usuari.setIdiomaUsuari(idioma).then(function(){
        $ionicLoading.show({template:$filter('translate')('lang_idioma_ok'), duration:1500});
      },function(error){
        $ionicLoading.show({template:$filter('translate')('lang_error_idioma'), duration:3500});
      });
    } else {
      $ionicLoading.show({template:$filter('translate')('lang_idioma_ok'), duration:1500});
    }
  } //Fi:canviarIdioma

  //TODO: DESCRIPCIO EVENT
  /**TODO
  * @ngdoc event
  * @name .# //the name after the hash is important
  * @eventOf . // the same part as above other than the hash
  * @eventType  on  // **`on`** is a keyword you can choose to either emit or broadcast on the target
  * @description En modificar l'idioma, canviem l'idioma de l'usuari i de l'aplicació
  */
  $scope.$watch('configuracio.idioma', function(newValue, oldValue){
    if(newValue !== $translate.use()){
      $scope.canviarIdioma(newValue);
    }
  }); //Fi:watch configuracio.idioma

}); // Fi:ConfigCtrl

/**
 * @ngdoc controller
 * @name app.controller:PerfilCtrl
 * @extends app.rootScope
 * @requires $scope
 * @requires $state
 * @requires $filter
 * @requires $ionicLoading
 * @requires $ionicPopup
 * @requires app.service:Usuari
 * @requires app.service:Auth
 * @requires app.service:Habitatges
 * @description
 * Controlador de la pàgina de modificació del perfil de l'usuari
 * @property {object} dadesUsuari Objecte que conté les dades de l'usuari actual
 * @property {boolean} editPassword Per habilitar l'edició de la contrasenya utilitzem aquest boolean
 * @property {boolean} editEmail Per habilitar l'edició de l'email utilitzem aquest boolean
 * @property {string} errorFormEmailFirebase Error retornat per firebase en el canvi de email
 * @property {string} errorFormPasswordFirebase Error retornat per firebase en el canvi de password
 * @property {string} errorFormPerfilFirebase Error retornat per firebase en el canvi de perfil
*/
app.controller('PerfilCtrl', function($scope, $ionicPopup, $ionicLoading, $state, $filter, Usuari, Auth, Habitatges) {
  $scope.editPassword = false;
  $scope.editEmail = false;
  //Part privada, si no està logejat anem al login
  if(!Auth.isLogged()){
    $state.go('app.login');
  }

  $scope.dadesUsuari = $scope.getCurrentUser();
  $scope.novesDadesUsuari = angular.copy($scope.dadesUsuari);

  /**
  * @ngdoc function
  * @name mostrarFormPassword
  * @methodOf app.controller:PerfilCtrl
  * @description Habilitem l'edició de la contrasenya
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.mostrarFormPassword = function(){
    $scope.editPassword = !$scope.editPassword;
  };

  /**
  * @ngdoc function
  * @name mostrarFormEmail
  * @methodOf app.controller:PerfilCtrl
  * @description Habilitem l'edició del correu electrònic de l'usuari
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.mostrarFormEmail = function(){
    $scope.editEmail = !$scope.editEmail;
  };


  /**
  * @ngdoc function
  * @name updateEmail
  * @methodOf app.controller:PerfilCtrl
  * @description Funció per modificar les dades del perfil de l'usauri
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.updateEmail = function(event){
      //Si s'ha canviat el email d'usuari l'actualitzem
    if($scope.novesDadesUsuari.data.email !== $scope.dadesUsuari.data.email){
      //Demanem la contrasenya a l'usuari
      var myPopup = $ionicPopup.show({
        template: '<input type="password" ng-model="novesDadesUsuari.data.password">',
        title: $filter('translate')('lang_password_requerida'),
        subTitle: $filter('translate')('lang_password_necessari'),
        scope: $scope,
        buttons: [
          { text: $filter('translate')('lang_cancelar') },
          {
            text: $filter('translate')('lang_ok'),
            type: 'button-positive',
            onTap: function(e) {
              if (!$scope.novesDadesUsuari.data.password) {
                e.preventDefault();
                $scope.errorFormEmailFirebase = 'lang_firebase_error_ERROR_GENERIC';
              } else {
                var novesDades = {
                  oldEmail: $scope.dadesUsuari.data.email,
                  newEmail: $scope.novesDadesUsuari.data.email,
                  password: $scope.novesDadesUsuari.data.password
                }
                Usuari.changeEmail(novesDades).then(function(resposta){
                  //Definim els valor d'alguns camps
                  if($scope.novesDadesUsuari.isVendor === false){
                    $scope.novesDadesUsuari.data.nivell = 2;
                  } else {
                    $scope.novesDadesUsuari.data.nivell = 3;
                  }
                  Usuari.setInfoUsuari($scope.novesDadesUsuari.data).then(function (resposta2){
                    //Tornem a assignar els valors
                    $scope.dadesUsuari = $scope.getCurrentUser();
                    $scope.novesDadesUsuari = angular.copy($scope.dadesUsuari);

                    //Mostrem un missatge de confirmació
                    var alertPopup = $ionicPopup.alert({
                      title: $filter('translate')('lang_modificacio_email'),
                      template: $filter('translate')('lang_modificacio_email_text'),
                      okText: $filter('translate')('lang_ok')
                    });

                    //Un cop actualitzades les dades, anem a la home
                    alertPopup.then(function(res) {
                      $state.go('app.perfil');
                    });
                  }, function(error){
                    $scope.errorFormEmailFirebase= 'lang_firebase_error_ERROR_GENERIC';
                  });
                } , function(error){
                  //Capturem l'error que retorna Firebase i el mostrem
                  switch (error.code) {
                    case "INVALID_PASSWORD":
                    case 'INVALID_USER':
                        $scope.errorFormEmailFirebase = 'lang_firebase_error_' + error.code;
                        break;
                    default:
                        $scope.errorFormEmailFirebase= 'lang_firebase_error_ERROR_GENERIC';
                  }
                });
              }
            }
          }
        ]
      });
    } else {
      $scope.errorFormEmailFirebase= 'lang_firebase_error_ERROR_GENERIC';
    }
  }; //Fi:updateEmail

  /**
  * @ngdoc function
  * @name updatePerfil
  * @methodOf app.controller:PerfilCtrl
  * @description Funció per modificar les dades del perfil de l'usauri
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.updatePerfil = function(event){
    event.preventDefault();

    //Mostrem l'animació de càrrega
    $ionicLoading.show({template:$filter('translate')('lang_load_actualitzant')});
    //Comprovem que disposem de totes les dades obligatòries
    //i que el canvi de email ha funcionat (si s'ha fet)
    if($scope.novesDadesUsuari.data.email != ''
      && $scope.novesDadesUsuari.data.nom){

      //Definim els valor d'alguns camps
      if($scope.novesDadesUsuari.isVendor === false){
        $scope.novesDadesUsuari.data.nivell = 2;
      } else {
        $scope.novesDadesUsuari.data.nivell = 3;
      }
      $scope.novesDadesUsuari.data.nom = $scope.novesDadesUsuari.data.nom;
      //Cridem al mètode corresponent a la modificació de dades
      Usuari.setInfoUsuari($scope.novesDadesUsuari.data).then(function(resposta){

        Usuari.setInfoActiveUsuari($scope.dadesUsuari.data.id, $scope.dadesUsuari.data.email).then(function(resposta2){
          //Tornem a assignar els scopes amb les noves dades
          $scope.dadesUsuari = $scope.getCurrentUser();
          $scope.novesDadesUsuari = angular.copy($scope.dadesUsuari);
        });

        //Mostrem un missatge de confirmació
        var alertPopup = $ionicPopup.alert({
          title: $filter('translate')('lang_modificacio_perfil'),
          template: $filter('translate')('lang_modificacio_perfil_text'),
          okText: $filter('translate')('lang_ok')
        });

        //Un cop actualitzades les dades, anem a la home
        alertPopup.then(function(res) {
          $state.go('app.perfil');
        });
      } , function(error){
        //Capturem l'error que retorna Firebase i el mostrem
        switch (error.code) {
          case "INVALID_PASSWORD":
          case 'INVALID_USER':
              $scope.errorFormPerfilFirebase = 'lang_firebase_error_' + error.code;
              break;
          default:
              $scope.errorFormPerfilFirebase= 'lang_firebase_error_ERROR_GENERIC';
        }
      });
    } else {
      $scope.errorFormPerfilFirebase = 'lang_firebase_error_ERROR_GENERIC';
    }

    //Amagem l'animació de càrrega
    $ionicLoading.hide();
  }; //Fi:updatePerfil


  /**
  * @ngdoc function
  * @name updatePassword
  * @methodOf app.controller:PerfilCtrl
  * @description Funció per actualitzar la contrasenya de l'usuari
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.updatePassword = function(event){
    event.preventDefault();

    //Mostrem l'animació de càrrega
    $ionicLoading.show({template:$filter('translate')('lang_load_actualitzant')});

    //Comprovem que disposem de totes les dades obligatòries
    if($scope.dadesUsuari.data.email != ''
      && $scope.novesDadesUsuari.data.password != ''
      && $scope.novesDadesUsuari.data.password == $scope.novesDadesUsuari.data.password2){
      var novesDades = {
        email: $scope.dadesUsuari.data.email,
        oldPassword: $scope.novesDadesUsuari.data.password_actual,
        newPassword: $scope.novesDadesUsuari.data.password,
      }
      //Cridem al mètode corresponent a la modificació de dades
      Usuari.changePassword(novesDades).then(function(resposta){
        //Mostrem un missatge de confirmació
        var alertPopup = $ionicPopup.alert({
          title: $filter('translate')('lang_modificacio_contrasenya'),
          template: $filter('translate')('lang_modificacio_contrasenya_text'),
          okText: $filter('translate')('lang_ok')
        });

        //Un cop actualitzades les dades, anem a la home
        alertPopup.then(function(res) {
          $state.go('app.home');
        });
      } , function(error){

        //Capturem l'error que retorna Firebase i el mostrem
        switch (error.code) {
          case "INVALID_PASSWORD":
          case 'INVALID_USER':
              $scope.errorFormPasswordFirebase = 'lang_firebase_error_' + error.code;
              break;
          default:
              $scope.errorFormPasswordFirebase= 'lang_firebase_error_ERROR_GENERIC';
        }
      });
    } else {
      $scope.errorFormPasswordFirebase = 'lang_firebase_error_ERROR_GENERIC';
    }

    //Amagem l'animació de càrrega
    $ionicLoading.hide();
  }; //Fi:updatePassword


  /**
  * @ngdoc function
  * @name removeUser
  * @methodOf app.controller:PerfilCtrl
  * @description Funció per eliminar el compte de l'usuari
  * @private
  * @returns {undefined} No retorna valors
  */
  $scope.removeUser = function(event){
    event.preventDefault();

    var deletePopup = $ionicPopup.show({
      template: '<input type="password" ng-model="dadesUsuari.data.password">',
      title: $filter('translate')('lang_password_requerida'),
      subTitle: $filter('translate')('lang_alerta_esborrar'),
      scope: $scope,
      buttons: [
        { text: $filter('translate')('lang_cancelar') },
        {
          text: $filter('translate')('lang_ok'),
          type: 'button-assertive',
          onTap: function(e) {
           if (!$scope.dadesUsuari.data.password) {
              e.preventDefault();
              $ionicLoading.show({template:$filter('translate')('lang_firebase_error_ERROR_GENERIC'), duration:2500});
            } else {
              var dataUserDelete = {
                  email: $scope.dadesUsuari.data.email,
                  password: $scope.dadesUsuari.data.password,
                  id: $scope.dadesUsuari.data.id
              };

              //Eliminem els habitatges de l'usuari
              Habitatges.deleteHabitatgesByUser(dataUserDelete.id).then(function(resposta){
                //Eliminem el compte de l'usuari
                Usuari.deleteUser(dataUserDelete).then(function(resposta2){
                  //Finalment destruim la sessió de l'usuaris
                  Auth.logout();
                  $state.go('app.home');
                }, function(error){
                  $ionicLoading.show({template:$filter('translate')('lang_firebase_error_ERROR_GENERIC'), duration:2500});
                });

              }, function(error){
                $ionicLoading.show({template:$filter('translate')('lang_firebase_error_ERROR_GENERIC'), duration:2500});
              });
            }
          }
        }
      ]
    });
  }; //Fi:removeUser

}); // Fi: PerfilCtrl