/*
------------------------------------
Directives
------------------------------------
*/
/**
 * @ngdoc directive
 * @name app.directive:pwCheck
 * @element input
 * @function link
 * @description
 * Comprova si coincideixen el valor dels dos inputs
 * Font: {@link https://github.com/forbode/forbode-angularjs-pwCheck pwCheck directive}
 * @example
   <example module="app">
     <file name="signup.html">
        <form name="formulari">
          <input ng-model="password" name="password" type="password" pw-check="password2" />
          <input ng-model="password2" name="password2" type="password" />
          <p class="error" ng-show="formulari.password.$error.pwmatch">No coincideixen</p>
          <p ng-show="!formulari.password.$error.pwmatch">Coincideixen</p>
        </form>
     </file>
   </example>
*/
app.directive('pwCheck', function () {
  return {
    require: 'ngModel',
    link: function (scope, elem, attrs, ctrl) {
      scope.$watch(attrs.pwCheck, function (confirmPassword) {
        var isValid = ctrl.$viewValue === confirmPassword;
        ctrl.$setValidity('pwmatch', isValid);
      });
    }
  }
});

/**
 * @ngdoc directive
 * @name app.directive:convertToNumber
 * @element select
 * @function link
 * @description
 * Passar a int els valors d'un select. Necessari per a seleccionar correctament els valors numérics d'un desplegable
 * Font: {@link https://code.angularjs.org/1.4.7/docs/api/ng/directive/select convertToNumber directive}
 * @example
   <example module="app">
     <file name="form-anunci.html">
        <form name="formulari">
          <select name="estat" ng-model="estat" required convert-to-number>
            <option ng-repeat="(key,item) in tipusAnunci" value="{{key}}">{{item | translate}}</option>
          </select>
        </form>
     </file>
   </example>
*/
app.directive('convertToNumber', function() {
  return {
    require: 'ngModel',
    link: function(scope, element, attrs, ngModel) {
      ngModel.$parsers.push(function(val) {
        return val ? parseInt(val, 10) : null;
      });
      ngModel.$formatters.push(function(val) {
        return val ? '' + val : null;
      });
    }
  };
});


/**
 * @ngdoc directive
 * @name app.directive:reverseGeocode
 * @element input
 * @function link
 * @restrict A
 * @description
 * Donades les coordenades de l'usuari, retorna una adreça obtinguda de Google Maps.
 * Basat en: {@link http://jasonwatmore.com/post/2014/02/15/AngularJS-Reverse-Geocoding-Directive.aspx Reverse Geocoding Directive}
 * @example
   <example module="app">
     <file name="home.html">
        <form name="formulari">
          <input type="search" placeholder="Cerca" ng-model="direccio" id="direccio" name="direccio"></input>
          <a reverse-geocode class="button button-calm icon ion-pinpoint margin-right"></a>
        </form>
     </file>
   </example>
*/
app.directive('reverseGeocode', function ($filter, $cordovaGeolocation) {
  return {
      restrict: 'A',
      replace: true,
      scope: false,
      link: function (scope, element, attrs) {
        element.bind('click', function(e) {
          e.preventDefault;
          scope.loading = true;
          var options = {timeout: 10000, enableHighAccuracy: true, maximumAge: 0};
          var geocoder = new google.maps.Geocoder();
          $cordovaGeolocation.getCurrentPosition(options).then(function(position){
            var latlng = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
            scope.data.lat = position.coords.latitude;
            scope.data.lng = position.coords.longitude;
            geocoder.geocode({ 'latLng': latlng }, function (results, status) {
              if (status == google.maps.GeocoderStatus.OK) {
                var resposta = $filter('translate')('lang_direccio_notfound');
                if (results[1]) {
                  resposta = results[1].formatted_address
                }
                scope.data.direccio = resposta;
                element.parent().children('#direccio').val(resposta);
                scope.$apply(function(){
                  scope.loading = false;
                });
              }

            }, function(error){
              angular.element.parent.children('#direccio').val($filter('translate')('lang_direccio_notfound'));
              scope.$apply(function(){
                scope.loading = false;
              });
            });
          }, function(error){
            angular.element.parent.children('#direccio').val($filter('translate')('lang_direccio_notfound'));
            scope.$apply(function(){
              scope.loading = false;
            });
          });
        });
      },
  }
});

/**
 * @ngdoc directive
 * @name app.directive:locationSuggestion
 * @function link
 * @restrict A
 * @description
 * Mostra una pantalla amb suggeriments d'adreces
 * obtingudes de Google Maps en funció del text introduït per l'usuari.
 * Retorna un objecte amb els components de l'adreça
 * Font: {@link http://devfanatic.com/blog/blog/2015/11/03/google-places-autocomplete-with-ionic-framework/ locationSuggestion}
 * @example
   <example module="app">
     <file name="home.html">
        <form name="formulari">
          <input type="text" name="search" ng-mode="search.query" location-suggestion />
        </form>
     </file>
   </example>
*/
app.directive('locationSuggestion', function($ionicModal, $filter, LocationService){
  return {
    restrict: 'A',
    scope: false,
    link: function($scope, element){
      $scope.search = {};
      $scope.search.suggestions = [];
      $scope.search.query = "";
      $ionicModal.fromTemplateUrl('templates/cercador_direccions.html', {
        scope: $scope,
        focusFirstInput: true
      }).then(function(modal) {
        $scope.modal = modal;
      });
      element[0].addEventListener('focus', function(event) {
        $scope.open();
      });
      $scope.$watch('search.query', function(newValue) {
        if (newValue) {
          LocationService.searchAddress(newValue).then(function(result) {
            $scope.search.error = null;
            $scope.search.suggestions = result;
          }, function(status){
            $scope.search.error = $filter('translate')('lang_no_results');
            $scope.search.suggestions = null;
          });
        };
        $scope.open = function() {
          $scope.search.suggestions = [];
          $scope.search.query = "";
          $scope.modal.show();
        };
        $scope.close = function() {
          $scope.modal.hide();
        };
        /*
        En seleccionar una de les opcions proposades, obtenim els detalls
        de l'adreça i els assignem a l'estructura de dades.
        */
        $scope.choosePlace = function(place) {
          LocationService.getDetails(place.place_id).then(function(location) {
            //Busquem els components de l'adreça per omplir el formulari
            $scope.search.query = location.formatted_address;
            if($scope.data){
              $scope.data.direccio = location.formatted_address;
              $scope.data.lat = location.geometry.location.lat();
              $scope.data.lng = location.geometry.location.lng();
              if(location.address_components){
                var components = location.address_components;
                for(var i in components){
                  if(components[i].types.indexOf('street_number') >= 0){
                    $scope.data.numero = parseInt(components[i].long_name);
                  }
                  if(components[i].types.indexOf('route') >= 0){
                    $scope.data.carrer = components[i].long_name;
                  }
                  if(components[i].types.indexOf('postal_code') >= 0){
                    $scope.data.cp = parseInt(components[i].long_name);
                  }
                  if(components[i].types.indexOf('locality') >= 0){
                    $scope.data.poblacio = components[i].long_name;
                  }
                  if(components[i].types.indexOf('country') >= 0){
                    $scope.data.pais = components[i].long_name;
                  }
                }
              }
            }
            $scope.close();
          });
        };
      });
    }
  }
});
