/*
------------------------------------
Serveis
------------------------------------
*/
/**
 * @ngdoc service
 * @name app.service:LocationService
 * @requires $q
 * @description
 * Servei que ens permet fer ús de les funcions de Google Places per autocompletar
 * adreces i obtenir-ne els detalls.
 * Font: {@link http://codepen.io/Froelund/pen/BuiDz}
**/
app.service('LocationService', function($q){
  var autocompleteService = new google.maps.places.AutocompleteService();
  var detailsService = new google.maps.places.PlacesService(document.createElement("input"));
  return {
    /**
     * @ngdoc method
     * @name app.service:LocationService#searchAdress
     * @methodOf app.service:LocationService
     * @description
     * Retorna suggeriments en funció de l'adreça introduida
     * @param {inputType} input L'input que crida al servei
     * @return {external:Promise} Si té éxit, la promesa conté un seguit d'adreces
     * Si hi ha un error, la promesa retorna l'status d'error de Google Places.
     */
    searchAddress: function(input) {
      var deferred = $q.defer();

      autocompleteService.getPlacePredictions({
        input: input
      }, function(result, status) {
        if(status == google.maps.places.PlacesServiceStatus.OK){
          deferred.resolve(result);
        }else{
          deferred.reject(status)
        }
      });

      return deferred.promise;
    }, //Fi:searchAddress

    /**
     * @ngdoc method
     * @name app.service:LocationService#getDetails
     * @methodOf app.service:LocationService
     * @description
     * Retorna sels detalls d'una adreça passada per id
     * @param {integer} placeId L'identificador de l'adreça
     * @return {external:Promise} Si té éxit, la promesa conté la informacíó de l'adreça
     * Si hi ha un error, la promesa retorna false.
     */
    getDetails: function(placeId) {
      var deferred = $q.defer();
      detailsService.getDetails({placeId: placeId}, function(result) {
        deferred.resolve(result);
      });
      return deferred.promise;
    } //Fi:getDetails
  };
})

/*
------------------------------------
Factory
------------------------------------
*/
/**
 * @ngdoc service
 * @name app.service:Habitatges
 * @requires $q
 * @requires app.URL_FIREBASE
 * @requires app.service:Usuari
 * @description
 * Els serveis relacionats amb l'accés a les dades dels habitatges estan
 * centralitzats en aquest Factory
 */
app.factory('Habitatges', function($q, URL_FIREBASE, Usuari) {
  var ref = new Firebase(URL_FIREBASE);
  //Mantenim dos objectes GeoFire per poder fer les consultes més rapidament i guardar-ne
  // els resultats coincidents a cada una de les branques
  var geoFirev = new GeoFire(ref.child('app/tipus-habitatge/venta/localitzacions'));
  var geoFirell = new GeoFire(ref.child('app/tipus-habitatge/lloguer/localitzacions'));
  var habitatgesServices = {

     /**
     * @ngdoc method
     * @name app.service:Habitatges#getInfoHabitatge
     * @methodOf app.service:Habitatges
     * @description
     * (Asíncrona) Retorna els detalls d'un habitatge fent una crida a Firebase
     * @param {string} key L'identificador de l'habitatge
     * @return {eternal:promise} Si té èxit, retorna la promesa conté un objecte
     * amb les dades de l'habitatge. Si no, el codi d'error de Firebase
     */
    getInfoHabitatge: function(key){
      var defered = $q.defer();
      ref.child('app/habitatges/').child(key).once("value").then(function(registre){
        var dadesHabitatge = registre.val();
        if(dadesHabitatge){
          dadesHabitatge.key = key;
          //Comprovem si és favorit de l'usuari
          Usuari.getInfoUsuariById(dadesHabitatge.id_usuari).then(function(dadesUsuariCreador){
            dadesHabitatge.dadesUsuari = dadesUsuariCreador;
            habitatgesServices.esHabitatgeFavorit(key).then(function(esFavorit){
              dadesHabitatge.esFavorit = esFavorit;
              defered.resolve(dadesHabitatge);
            },function(error){
              defered.reject(error);
            });
          }, function(error){
            defered.reject(error);
          });
        } else {
          defered.reject(false);
        }
      }, function (error) {
        defered.reject(error);
      });
      return defered.promise;
    }, //Fi:getInfoHabitatge

    /**
    * @ngdoc method
    * @name app.service:Habitatges#getHabitatgesByUser
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Retorna el llistat d'habitatges que ha creat l'usuari especificat. Fa una crida a Firebase
    * @param {string} idUsuari L'identificador de l'usuari
    * @return {external:Promise} Si té éxit, la promesa conté una col·lecció d'objectes amb els habitatges de l'usuari
    * Si hi ha un error, la promesa retorna l'status d'error de Firebase
    */
    getHabitatgesByUser: function(idUsuari){
      var defered = $q.defer();
      ref.child('app/users/' + idUsuari + '/habitatges').once("value",function(registre){
        var habitatgeCol = registre.val();
        defered.resolve(habitatgeCol);
      }, function (error) {
        defered.reject(error);
      });

      return defered.promise;
    }, //Fi:getHabitatgesByUser

    /**
    * @ngdoc method
    * @name app.service:Habitatges#deleteHabitatgesByUser
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Elimina els habitatges de l'usuari de les taules d'habitatges i localitzacions.
    * Fa ús del mètode getHabitatgesByUser i deleteHabitatge, d'aquest mateix Factory.
    * @param {string} idUsuari L'identificador de l'usuari
    * @return {external:Promise} Si té éxit, la promesa retorna true
    * Si hi ha un error, la promesa retorna l'status d'error de Firebase
    */
    deleteHabitatgesByUser: function(idUsuari){
      var defered = $q.defer();
      //Eliminem la resta de dades de l'aplicació (habitatges
      //i informació del compte)
      this.getHabitatgesByUser(idUsuari).then(function(habitatgesUsuari){
        angular.forEach(habitatgesUsuari, function(valor, element){
          ref.child('app/habitatges').child(element).once("value").then(function(hab){
            var habDelete = hab.val();
            habitatgesServices.deleteHabitatge(element, habDelete.tipus);
          }, function(error){
            defered.reject(error);
          });
          defered.resolve(true);
        });
      }, function(error){
        defered.reject(error);
      });
      return defered.promise;
    }, //Fi:deleteHabitatgeByUser

    /**
    * @ngdoc method
    * @name app.service:Habitatges#deleteHabitatge
    * @methodOf app.service:Habitatges
    * @description
    * Elimina un habitatge de la taula d'habitatges i localitzaions.
    * @param {string} idHabitatge L'identificador de l'habitatge
    * @param {string} tipus El tipus d'habitatge. Necessari per esborrar-lo de la branca corresponent de localitzacions.
    * @return {undefined} No retorna dades
    */
    deleteHabitatge: function(idHabitatge, tipus){
      ref.child('app/habitatges/' + idHabitatge).set(null);
      ref.child('app/tipus-habitatge/' + tipus + '/localitzacions/' + idHabitatge).set(null);
    }, //Fi:deleteHabitatge

    /**
    * @ngdoc method
    * @name app.service:Habitatges#incrementarVisualitzacions
    * @methodOf app.service:Habitatges
    * @description
    * Incrementa el número de visualitzacions d'un habitatge.
    * @param {string} key L'identificador de l'habitatge
    * @return {integer} El número de visualitzacions
    */
    incrementarVisualitzacions: function(key){
      ref.child('app/habitatges/' + key + '/num_visualitzacions/').transaction(function (numVisualitzacions){
        return numVisualitzacions+1;
      });
    }, //Fi:incrementarVisualitzacions

    /**
    * @ngdoc method
    * @name app.service:Habitatges#setFieldHabitatge
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Estableix el valor d'un camp de l'habitatge a la taula d'habitatges de la base de dades
    * @param {string} key L'identificador de l'habitatge
    * @param {string} field El camp a modificar
    * @param {undefined} nouValor El nou valor del camp
    * @return {external:Promise} Si té éxit, la promesa retorna true
    * Si hi ha un error, la promesa retorna false
    */
    setFieldHabitatge: function(key, field, nouValor){
      var defered = $q.defer();
      if(key && field){
        ref.child("app/habitatges/" + key + '/' + field).set(nouValor).then(function(){
            defered.resolve(true);
        }, function(error){
            defered.reject(false);
        })
      } else {
        defered.reject(false);
      }
      return defered.promise;
    }, //Fi:setFieldHabitatge

    /**
    * @ngdoc method
    * @name app.service:Habitatges#updateHabitatge
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Actualitza les dades d'un habitatge
    * @param {object} novesDades Objecte que conté les dades de l'habitatge
    * @param {array} colFotos Array d'objectes amb les dades de fotografies de l'habitatge (en base64)
    * @return {external:Promise} Si té éxit, la promesa retorna true
    * Si hi ha un error, la promesa retorna false o el codi d'error de Firebase
    */
    updateHabitatge: function(novesDades, colFotos){
      var defered = $q.defer();
      var usuariActiu = Usuari.getActiveUser();
      if(usuariActiu.data.id){
        var habitatgeId = novesDades.key;
        var habitatgeFirebase = ref.child('app/habitatges/' + habitatgeId);
        if(habitatgeFirebase){
          //Abans de fer l'update mirem si l'hem de canviar de categoria
          if(habitatgeFirebase.tipus != novesDades.tipus){

            if(novesDades.tipus == 'venta'){
              geoFireUpdate = geoFirev;
            } else {
              geoFireUpdate = geoFirell;
            }
            //L'eliminem de l'actual categoria i l'afegim a la nova
            ref.child('/app/tipus-habitatge/' + habitatgeFirebase.tipus  + '/localitzacions/' + novesDades.key).remove();
            geoFireUpdate.set(novesDades.key, [novesDades.lat, novesDades.lng]).then(function(){
            }, function(error) {
              defered.reject(error);
            });
          } else {
            //Si no l'hem de moure de categoria, n'actualitzem les coordenades
            ref.child('/app/tipus-habitatge/' + novesDades.tipus  + '/localitzacions/' + novesDades.key).update(novesDades.key, [novesDades.lat, novesDades.lng]);
          }
          //Actualitzem totes dades de l'habitatge a la taula principal d'habitatges
          habitatgeFirebase.update(novesDades)
          //Pugem les noves fotos (si hi ha canvis)
          if(colFotos.length > 0){
            angular.forEach(colFotos, function(data, key) {
              if(!data.data64){ // Si el camp és buit, esborrem la foto de la base de dades
                ref.child("app/habitatges/" + habitatgeId + '/imatges/' + data.id).remove();
              } else {
                ref.child("app/habitatges/" + habitatgeId + '/imatges/' + data.id).set(data.data64);
              }
            });
          } else {
            //Esborrem totes les fotos si ens arriba un array buit
            ref.child("app/habitatges/" + habitatgeId + '/imatges/').remove();
          }
          defered.resolve(true);
        }else {
          defered.reject(false);
        }
      } else {
        defered.reject(false);
      }
      return defered.promise;
    }, //Fi:updateHabitatge

    /**
    * @ngdoc method
    * @name app.service:Habitatges#createHabitatge
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Afegeix un nou habitatge a la base de dades. Ho fa a les taules habitatges i localitzacions
    * @param {object} dadesHabitatge Objecte que conté les dades de l'habitatge
    * @param {array} colFotos Array d'objectes amb les dades de fotografies de l'habitatge (en base64)
    * @return {external:Promise} Si té éxit, la promesa retorna l'identificador de l'habitatge
    * Si hi ha un error, la promesa retorna false o el codi d'error de Firebase
    */
    createHabitatge: function(dadesHabitatge, colFotos){
      var defered = $q.defer();
      var usuariActiu = Usuari.getActiveUser();

      if(usuariActiu.data.id){
        var nouHabitatge = ref.child("app/habitatges").push(dadesHabitatge
        , function (error){
          defered.reject(error);
        });

        if(dadesHabitatge.tipus == 'venta'){
          geoFireActive = geoFirev;
        } else {
          geoFireActive = geoFirell;
        }

        var habitatgeId = nouHabitatge.key();
        if(habitatgeId){
          //Afegim l'id de l'habitatge a la col·lecció
          //d'habitatges de l'usuari
          ref.child('app/users').child(usuariActiu.data.id).child('habitatges').child(habitatgeId).set(true);

          if(colFotos.length > 0){
            angular.forEach(colFotos, function(data, key) {
              if(data.data64){ //Comprovem que la foto tingui dades
                ref.child("app/habitatges/" + habitatgeId + '/imatges/').push(data.data64
                , function (error){
                  defered.reject(error);
                });
              }
            });
          }
          //Afegim les coordenades de l'habitatge a la col·lecció
          //de coordenades que utilitza GeoFire, a la branca que toca (lloguer, venta)
          geoFireActive.set(habitatgeId, [dadesHabitatge.lat, dadesHabitatge.lng]).then(function() {
          }, function(error) {
            defered.reject(error);
          });
          defered.resolve(habitatgeId);
        } else {
          defered.reject(false);
        }
      } else {
        defered.reject(false);
      }
      return defered.promise;
    }, //Fi:createHabitatge

    /**
    * @ngdoc method
    * @name app.service:Habitatges#Habitatges#addFavorit
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Marca un habitatge com a favorit de l'usuari actiu.
    * @param {string} key L'identificador de l'habitatge
    * @return {external:Promise} Si té éxit, la promesa retorna true
    * Si hi ha un error, la promesa retorna false
    */
    addFavorit: function(key){
      var defered = $q.defer();
      var usuariActiu = Usuari.getActiveUser();
      if(key && usuariActiu){
        ref.child('app/users/' + usuariActiu.data.id + '/favorits/' + key).set(true);
        ref.child('app/habitatges/' + key + '/num_favorits').transaction(function (numFavorits){
          return numFavorits+1;
        });
        defered.resolve(true);
      }else {
        defered.reject(false);
      }
      return defered.promise;
    }, //Fi:addFavorit

    /**
    * @ngdoc method
    * @name app.service:Habitatges#removeFavorit
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Desmarca un habitatge com a favorit de l'usuari actiu.
    * @param {string} key L'identificador de l'habitatge
    * @return {external:Promise} Si té éxit, la promesa retorna true
    * Si hi ha un error, la promesa retorna false
    */
    removeFavorit: function(key){
      var defered = $q.defer();
      var usuariActiu = Usuari.getActiveUser();
      if(key && usuariActiu){
        ref.child('app/users/' + usuariActiu.data.id + '/favorits/' + key).remove();
        ref.child('app/habitatges/' + key + '/num_favorits').transaction(function (numFavorits){
          return numFavorits-1;
        });
        defered.resolve(true);
      }else {
        defered.reject(false);
      }
      return defered.promise;
    }, //Fi:removeFavorit

    /**
    * @ngdoc method
    * @name app.service:Habitatges#esHabitatgeFavorit
    * @methodOf app.service:Habitatges
    * @description
    * (Asíncrona) Funció per saber si un habitatge és favorit de l'usuari actiu.
    * @param {string} key L'identificador de l'habitatge
    * @return {external:Promise} Si té éxit, la promesa retorna true
    * Si hi ha un error, la promesa retorna false
    */
    esHabitatgeFavorit: function(key){
      var defered = $q.defer();
      var usuariActiu = Usuari.getActiveUser();
      if(key && usuariActiu){
        ref.child('app/users/' + usuariActiu.data.id + '/favorits/' + key).once("value", function(habitatge){
          defered.resolve(habitatge.exists());
        }, function(error){
          defered.reject(false);
        });
      } else {
        defered.reject(false);
      }
      return defered.promise;
    }, //Fi:esHabitatgeFavorit
  };

  return habitatgesServices;
});

/**
 * @ngdoc service
 * @name app.service:GoogleMaps
 * @requires $q
 * @description
 * Utilitats i serveis relacionats amb l'accés al servei de Google Maps i la
 * creació de mapes a l'aplicació
 * @property {array} maps Array de mapes utilitzats a l'aplicació
**/
app.factory("GoogleMaps", function ($q) {
  var maps = {};

  var mapServices = {
    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#addMap
    * @methodOf app.service:GoogleMaps
    * @description
    * Afegim un mapa a la col·lecció de mapes
    * @param {string} idMap L'identificador del mapa
    * @param {object} map L'objecte de GoogleMaps que conté el mapa
    * @return {undefined} No retorna dades
    */
    addMap: function(idMap, map){
      if(!maps[idMap]){
        maps[idMap] = {map: map, markers: []};
      }
    }, //Fi:addMap

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#getMap
    * @methodOf app.service:GoogleMaps
    * @description
    * Retorna l'objecte que conté les dades i el mapa
    * @param {string} idMap L'identificador del mapa dins l'array
    * @return {object} Objecte amb les dades del mapa
    */
    getMap: function(idMap){
      return maps[idMap];
    }, //Fi:getMap

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#setMap
    * @methodOf app.service:GoogleMaps
    * @description
    * Desem l'objecte map de GoogleMaps a l'array de mapes
    * @param {string} idMap L'identificador del mapa dins l'array
    * @param {object} map L'objecte de GoogleMaps que conté el mapa
    * @return {object} Objecte amb les dades del mapa de GoogleMaps
    */
    setMap: function(idMap, map){
      return maps[idMap].map = map;
    }, //Fi:setMap

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#iniMap
    * @methodOf app.service:GoogleMaps
    * @description
    * Invoquem la funció d'inicialització del mapa de GoogleMaps que el mostra per pantalla
    * @param {string} idMap L'identificador del mapa dins l'array.
    * @param {float} lat Latitud del centre del mapa.
    * @param {float} lat Lognitud del centre del mapa.
    * @param {integer} zoom Zoom del mapa.
    * @param {boolean} draggable Indiquem si el mapa es pot arrossegar o no.
    * @param {boolean} zoomable Indiquem si es pot fer zoom al mapa o no.
    * @return {object} Objecte mapa de GoogleMaps
    */
    iniMap: function(idMap, lat, lng, zoom, draggable, zoomable){
      if(maps[idMap]){
        var latLngIni = new google.maps.LatLng(lat, lng);
        var mapOptions = {
          center: latLngIni,
          zoom: zoom,
          mapTypeId: google.maps.MapTypeId.ROADMAP,
          streetViewControl: false,
          draggable: draggable,
          scrollwheel : zoomable
        };
        var map = new google.maps.Map(document.getElementById('map'), mapOptions);
        this.setMap(idMap, map);
      }
      return this.getMap(idMap).map;
    }, //Fi:iniMap

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#setCenter
    * @methodOf app.service:GoogleMaps
    * @description
    * Establim el centre d'un mapa
    * @param {string} idMap L'identificador del mapa dins l'array.
    * @param {object} location Objecte amb la latitud i longitud del centre.
    * @return {undefined} No retorna dades
    */
    setCenter: function (idMap, location){
      var map = this.getMap(idMap).map;
      var latLng = new google.maps.LatLng(location.lat, location.lng);
      if(map){
        map.setCenter(latLng);
      }
    }, //Fi:setCenter

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#addMarker
    * @methodOf app.service:GoogleMaps
    * @description
    * Afegim un marcador a un mapa (el mostra per pantalla)
    * @param {string} idMap L'identificador del mapa.
    * @param {object} location Objecte amb la latitud i longitud del marcador.
    * @param {string} key L'identificador del marcador
    * @return {object} L'objecte marcador de GoogleMaps
    */
    addMarker : function(idMap, location, key){

      //Afegim un marcador al Mapa
      var latLng = new google.maps.LatLng(location.lat, location.lng);
      var map = this.getMap(idMap).map;
      var marker = new google.maps.Marker({
          position: latLng,
          draggable: false,
          icon: key != 'posicio' ? './img/ico_casa.png' : ''
      });
      var existeix = maps[idMap].markers.indexOf(key);
      if(existeix == -1){
        marker.setMap(map);
        maps[idMap].markers.push({key: key, marker: marker});
        return marker;
      } else {
        return maps[idMap].markers[existeix];
      }
    }, //Fi:addMarker

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#removeMarker
    * @methodOf app.service:GoogleMaps
    * @description
    * Amaguem un marcador d'un mapa
    * @param {string} idMap L'identificador del mapa.
    * @param {string} key L'identificador del marcador
    * @return {undefined} No retorna dades
    */
    removeMarker : function(idMap, key){
      var mapObj = this.getMap(idMap);
      for (var i = 0; i < mapObj.markers.length; i++) {
        var markerObj = mapObj.markers[i];
        if(markerObj.key == key){
          mapObj.markers[i].marker.setMap(null);
          mapObj.markers.splice(i,1);
          continue;
        }
      }
    }, //Fi:removeMarker

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#addAllMarkers
    * @methodOf app.service:GoogleMaps
    * @description
    * Mostrem per pantalla tots els marcadors d'un mapa
    * @param {string} idMap L'identificador del mapa.
    * @return {undefined} No retorna dades
    */
    addAllMarkers : function(idMap){
      var mapObj = this.getMap(idMap);
      if(mapObj && mapObj.markers){
        for (var i = 0; i < mapObj.markers.length; i++) {
          var markerColObj = mapObj.markers[i]
          markerColObj.marker.setMap(mapObj.map);
        }
      }
    }, //Fi:addAllMarkers

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#removeAllMarkers
    * @methodOf app.service:GoogleMaps
    * @description
    * Amaguem tots els marcadors d'un mapa
    * @param {string} idMap L'identificador del mapa.
    * @return {undefined} No retorna dades
    */
    removeAllMarkers : function(idMap){
      var mapObj = this.getMap(idMap);
      if(mapObj && mapObj.markers){
        for (var i = 0; i < mapObj.markers.length; i++) {
          var markerColObj = mapObj.markers[i]
          markerColObj.marker.setMap(null);
        }
      }
    }, //Fi:removeAllMarkers

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#resetMarkers
    * @methodOf app.service:GoogleMaps
    * @description
    * Esborrem tots els marcadors d'un mapa
    * @param {string} idMap L'identificador del mapa.
    * @return {undefined} No retorna dades
    */
    resetMarkers : function(idMap){
      var mapObj = this.getMap(idMap);
      if(mapObj){
        mapObj.markers = [];
      }
    }, //Fi:resetMarkers

    /**
    * @ngdoc method
    * @name app.service:GoogleMaps#addSearchCircle
    * @methodOf app.service:GoogleMaps
    * @description
    * Afegim un cercle que representa el radi de cerca a un mapa (el mostrem per pantalla)
    * @param {string} idMap L'identificador del mapa.
    * @param {float} lat Latitud del centre del cercle.
    * @param {float} lat Lognitud del centre del cercle.
    * @param {float} radi El radi (en quilòmetres) del cercle
    * @param {boolean} color1 Color del marc del cercle
    * @param {boolean} color2 Color de fons del cercle
    * @return {object} Objecte poligon de google maps
    */
    addSearchCircle : function(idMap, lat, lng, radi, color1, color2){
      var latLng = new google.maps.LatLng(lat, lng);
      var map = this.getMap(idMap).map;
      var cercleCerca = new google.maps.Circle({
        strokeColor: color1,
        strokeOpacity: 0.6,
        strokeWeight: 2,
        fillColor: color2,
        fillOpacity: 0.35,
        map: map,
        center: latLng,
        radius: radi * 1000,
        draggable: true
      });
      return cercleCerca;
    }, //Fi:addSearchCircle
  }

  return mapServices;
});

/**
 * @ngdoc service
 * @name app.service:Auth
 * @requires app.URL_FIREBASE
 * @requires app.service:Usuari
 * @requires $q
 * @description
 * Els serveis relacionats amb la identificació d'usuaris estan
 * centralitzats en aquest Factory
 * @property {object} ref Objecte Firebase
**/
app.factory("Auth", function (URL_FIREBASE, Usuari, $q) {
  var ref = new Firebase(URL_FIREBASE);
  var authServices = {
    /**
    * @ngdoc method
    * @name app.service:Auth#login
    * @methodOf app.service:Auth
    * @description
    * (Asíncrona) Identifiquem un usuari a l'aplicació mitjançant Firebase.
    * @param {object} credencials Objecte que conté les dades d'accéss de l'usuari
    * @return {external:promise} Si té èxit, retorna un objecte amb les dades de l'usuari
    * Si no, retorna el codi d'error de Firebase
    */
    login : function(credencials){
      var defered = $q.defer();
      ref.authWithPassword({
        email: credencials.email,
        password: credencials.password
      }, function(error, authData) {
        if (error) {
          defered.reject(error);
        } else {
          //Guardem la data del login per comprovar si hi ha novetats
          ref.child('app/users/' + authData.uid + '/lastLogin').set(new Date().getTime());
          defered.resolve(authData);
        }
      });
      return defered.promise;
    }, // Fi:login

    /**
    * @ngdoc method
    * @name app.service:Auth#logout
    * @methodOf app.service:Auth
    * @description
    * Deslogem l'usuari de firebase i en destruim les dades de la sessió.
    * @return {undefined} No retorna dades
    */
    logout : function(){
      ref.unauth();
      Usuari.destroyActiveUser();
    }, // logout

    /**
    * @ngdoc method
    * @name app.service:Auth#isLogged
    * @methodOf app.service:Auth
    * @description
    * Invoquem al mètode getAuth de Firebase per comprovar que un usuari està logejat.
    * @return {boolean} Si té èxit, la promesa val true
    * Si no, retorna false.
    */
    isLogged : function(){
      var authData = ref.getAuth();
      var activeUser = Usuari.getActiveUser();
      if(authData && activeUser.isLogged){
        return true;
      }
      return false;
    }
  }
  return authServices;
});


/**
 * @ngdoc service
 * @name app.service:Usuari
 * @requires app.URL_FIREBASE
 * @requires $q
 * @description
 * Els serveis relacionats amb les dades dels usuaris estan
 * centralitzats en aquest Factory. A més, desa les dades de l'usuari actiu
 * @property {Usuari} activeUser Dades de l'usuari actiu de l'aplicació
**/
app.factory("Usuari", function (URL_FIREBASE, $q) {
  var ref = new Firebase(URL_FIREBASE);
  var activeUser = {
    isLogged: false,
    isVendor: false,
    data: {}
  };

  // Interfície per accedir a les funcions de Firebase
  var usersServices = {
    /**
    * @ngdoc method
    * @name app.service:Usuari#getActiveUser
    * @methodOf app.service:Usuari
    * @description
    * Retorna les dades de l'usuari actiu
    * @return {object} Objecte que conté les dades de l'usuari actiu
    */
    getActiveUser: function(){
      return activeUser;
    }, //Fi:getActiveUser

    /**
    * @ngdoc method
    * @name app.service:Usuari#getInfoUsuariById
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Retorna les dades d'un usuari desades a la base de dades
    * @param {string} idUsuari L'identificador de l'usuari
    * @return {external:promise} Objecte que conté les dades de l'usuari actiu
    */
    getInfoUsuariById: function(idUsuari){
      var defered = $q.defer();
      if(idUsuari){
        ref.child('app/users').child(idUsuari).on("value", function(registre) {
          dadesUsuari = registre.val();
          defered.resolve(dadesUsuari);
        }, function(error){
          defered.reject(error);
        });
      } else {
        defered.reject(null);
      }
      return defered.promise;
    }, //Fi:getInfoUsuariById

    /**
    * @ngdoc method
    * @name app.service:Usuari#setInfoActiveUsuari
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Desa a l'objecte usuariActiu les dades de l'usuari actual obtingudes de la base de dades
    * @param {string} idUsuari L'identificador de l'usuari
    * @param {string} email El correu electrònic (username) de l'usuari
    * @return {external:promise} Si té èxit, la promesa retorna true
    * si no, retorna el codi d'error de Firebase
    */
    setInfoActiveUsuari : function(idUsuari, email){
      var defered = $q.defer();
      ref.child('app/users').child(idUsuari).on("value",function(registre) {
        dadesUsuari = registre.val();
        if(dadesUsuari){
          activeUser.data.id = idUsuari;
          activeUser.data.nom = dadesUsuari.nom;
          activeUser.data.telefon = dadesUsuari.telefon;
          activeUser.data.email = dadesUsuari.email;
          activeUser.data.nivell = dadesUsuari.nivell;
          activeUser.data.idioma = dadesUsuari.idioma;
          activeUser.data.showPhone = dadesUsuari.showPhone;
          activeUser.data.showEmail = dadesUsuari.showEmail;
          activeUser.isLogged = true;
          if(dadesUsuari.nivell === 3){
            activeUser.isVendor = true;
          }
          defered.resolve(true);
        } else {
          defered.reject(false);
        }
      }, function(error){
        defered.reject(error);
      });
      return defered.promise;
    }, //Fi:setInfoActiveUsuari

    /**
    * @ngdoc method
    * @name app.service:Usuari#setInfoUsuari
    * @methodOf app.service:Usuari
    * @description
    * Desa a la base de dades les dades de l'usuari
    * @param {object} dadesUsuari Les dades de l'usuari actiu
    * @return {undefined} No retorna dades
    */
    setInfoUsuari : function(dadesUsuari){
      return ref.child("app/users").child(dadesUsuari.id).set({
        nom: dadesUsuari.nom,
        telefon: dadesUsuari.telefon,
        nivell: dadesUsuari.nivell,
        showPhone: dadesUsuari.showPhone,
        showEmail: dadesUsuari.showEmail,
        email: dadesUsuari.username,
        idioma: dadesUsuari.idioma
      });
    }, //Fi:setInfoUsuari

    /**
    * @ngdoc method
    * @name app.service:Usuari#setIdiomaUsuari
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Desa a la base de dades l'idioma seleccionat de l'usuari en que es
    * mostrarà l'aplicació en les succesives sessions.
    * @param {string} idioma L'identificador de l'idioma
    * @return {undefined} No retorna dades
    */
    setIdiomaUsuari: function(idioma){
      return ref.child("app/users").child(activeUser.data.id + '/idioma').set(idioma);
    }, //Fi:setIdiomaUsuari

    /**
    * @ngdoc method
    * @name app.service:Usuari#createUser
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Afegeix un usuari a Firebase
    * @param {object} credencials Objecte que conté les credencials del nou usuari
    * @return {external:promise} Si té èxit, credencials retorna un objecte amb les dades de l'usuari
    * si no, retorna un codi d'error de Firebase
    */
    createUser : function(credencials){
      var defered = $q.defer();
      ref.createUser({
        email: credencials.username,
        password: credencials.password
      }, function(error, authData) {
        if (error) {
          defered.reject(error);
        } else {
          defered.resolve(authData);
        }
      });
      return defered.promise;
    }, //Fi:createUser

    /**
    * @ngdoc method
    * @name app.service:Usuari#deleteUser
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Elimina un usuari de la base de dades de Firebase
    * @param {object} credencials Objecte que conté les credencials de l'usuari a esborrar usuari
    * @return {external:promise} Si té èxit, la promesa retorna true
    * si no, retorna un codi d'error de Firebase
    */
    deleteUser : function(credencials){
      var defered = $q.defer();
      ref.removeUser({
        email:credencials.email,
        password: credencials.password
      }, function(error, authData) {
        if (error) {
          defered.reject(error);
        } else {
          ref.child('app/users/' + credencials.id).set(null);
          defered.resolve(true);
        }
      });
      return defered.promise;
    }, //Fi:deleteUser

    /**
    * @ngdoc method
    * @name app.service:Usuari#changePassword
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Canviem la contrasenya de l'usuari actiu
    * @param {object} dadesPasswords Objecte que conté la nova contrasenya i les credencials de l'usuari
    * @return {external:promise} Si té èxit, la promesa retorna true
    * si no, retorna un codi d'error de Firebase
    */
    changePassword: function(dadesPasswords){
      var defered = $q.defer();
      ref.changePassword(dadesPasswords, function(error) {
        if (error) {
          defered.reject(error);
        } else {
          defered.resolve(true)
        }
      });
      return defered.promise;
    }, //Fi:changePassword

    /**
    * @ngdoc method
    * @name app.service:Usuari#changeEmail
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Canviem el correu electrònic (username a firebase) de l'usuari actiu
    * @param {object} dadesPasswords Objecte que conté la nova contrasenya i les credencials de l'usuari
    * @return {external:promise} Si té èxit, la promesa retorna true
    * si no, retorna un codi d'error de Firebase
    */
    changeEmail: function(dadesEmails){
      var defered = $q.defer();
      ref.changeEmail(dadesEmails, function(error) {
        if (error) {
          defered.reject(error);
        } else {
          defered.resolve(true)
        }
      });
      return defered.promise;
    }, //Fi:changeEmail

    /**
    * @ngdoc method
    * @name app.service:Usuari#resetPassword
    * @methodOf app.service:Usuari
    * @description
    * (Asíncrona) Invoca el mètode resetPassword de Firebase per enviar una nova contrasenya per correu a l'usuari
    * @param {string} email Adreça de correu (username a Firebase) de l'usauri que vol recuperar la contrasenya.
    * @return {external:promise} Si té èxit, la promesa retorna true
    * si no, retorna un codi d'error de Firebase
    */
    resetPassword : function(email){
      var defered = $q.defer();
      ref.resetPassword({
        email: email
      }, function(error) {
        if (error) {
          defered.reject(error);
        } else {
          defered.resolve(true);
        }
      });
      return defered.promise;
    }, //Fi:resetPassword

    /**
    * @ngdoc method
    * @name app.service:Usuari#destroyActiveUser
    * @methodOf app.service:Usuari
    * @description
    * Esborrem les dades de l'usuari actual
    * @return {undefined} No retorna dades
    */
    destroyActiveUser: function(){
      //Netejem les dades de l'usuari
      activeUser.data = {};
      activeUser.isLogged = false;
      activeUser.isVendor = false;
    }
  }; // Fi Objecte usuaris
  return usersServices;
});

/**
 * @ngdoc service
 * @name app.service:Cerca
 * @requires app.service.GoogleMaps
 * @requires $rootScope
 * @description
 * Els serveis relacionats amb la cerca de l'aplicació
 * centralitzats en aquest Factory. A més, desa les dades de la cerca i els
 * seus resultats.
 * @property {Cerca} cercaModel Model de dades de la cerca
 * @property {Cerca} cerca Dades de la cerca actual, segueix el model de cercaModel
**/
app.factory("Cerca", function (GoogleMaps, $rootScope) {
  //Valors per defecte de la cerca
  var cercaModel = {
    tipusCerca: "venta",
    direccio: '',
    lat: 0,
    lng: 0,
    maxPreu: 0,
    minPreu: 0,
    minHab: 0,
    maxHab: 0,
    minLav: 0,
    maxLav: 0,
    minSup: 0,
    maxSup: 0,
    estat_habitatge: 99,
    teParquing: null,
    teTerrassa: null,
    tePiscina: null,
    teCalefaccio: null,
    teMobles: null,
    teAscensor: null,
    teElectrodomestics: null,
    teAireCondicionat: null,
    tePati: null,
    teBalcons: null,
    ordenacio: null,
    radi: 1.5,
    zoom: 14,
    habitatges: [],
    novaCerca: false //Utilitzat per forçar una nova cerca
  };

  var cerca = angular.copy(cercaModel);

  return {
    /**
    * @ngdoc method
    * @name app.service:Cerca#setDadesCerca
    * @methodOf app.service:Cerca
    * @description
    * Estableix l'objecte cerca amb noves dades
    * @param {objecte} data Objecte amb les dades de cerca
    * @return {undefined} No retorna dades
    */
    setDadesCerca: function(data) {
      cerca = data;
    }, //Fi:setDadesCerca

    /**
    * @ngdoc method
    * @name app.service:Cerca#getDadesCerca
    * @methodOf app.service:Cerca
    * @description
    * Obtenim l'objecte que conté les dades de la cerca actual.
    * @return {objecte} cerca Objecte amb les dades de la cerca actual.
    */
    getDadesCerca: function() {
       return cerca;
    }, //FigetDadesCerca

    /**
    * @ngdoc method
    * @name app.service:Cerca#addHabitatge
    * @methodOf app.service:Cerca
    * @description
    * @param {object} hab Objecte que conté les dades d'un habitatge
    * Afegim un objecte que conté les dades d'un habitatge a l'array de resultats de la cerca
    * @return {undefined} No retorna dades
    */
    addHabitatge: function(hab){
      cerca.habitatges.push(hab);
    }, //Fi:addHabitatge

    /**
    * @ngdoc method
    * @name app.service:Cerca#removeHabitatge
    * @methodOf app.service:Cerca
    * @description
    * Eliminem un habitatge de l'array de resultats de la cerca
    * @param {string} key L'identificador de l'habitatge a Firebase
    * @return {undefined} No retorna dades
    */
    removeHabitatge: function(key){
      var i = this.getHabitatgeIndexByKey(key);
      if(i !== null){
        cerca.habitatges.splice(i, 1);
      }
    }, //Fi:removeHabitatge

    /**
    * @ngdoc method
    * @name app.service:Cerca#getHabitatgeIndexByKey
    * @methodOf app.service:Cerca
    * @description
    * Obtenim l'index dins l'array de resultats d'un habitatge de la cerca a partir del seu identificador de Firebase
    * @param {string} key L'identificador de l'habitatge a Firebase
    * @return {integer} L'índex de l'habitatge a l'array de resultats
    */
    getHabitatgeIndexByKey: function(key){
      var index = null;

      for(var i=0; i<cerca.habitatges.length;i++){
        if(cerca.habitatges[i].key == key){
          index = i;
          break;
        }
      }
      return index;
    }, //Fi:getHabitatgeIndexByKey

    /**
    * @ngdoc method
    * @name app.service:Cerca#clearResultats
    * @methodOf app.service:Cerca
    * @description
    * Eliminem tots els resultats de la cerca
    * @return {undefined} No retorna dades
    */
    clearResultats: function(){
      cerca.habitatges = [];
    }, //Fi:clearResultats

    /**
    * @ngdoc method
    * @name app.service:Cerca#netejaFiltres
    * @methodOf app.service:Cerca
    * @description
    * Esborrem tots els possibles filtres aplicats
    * @return {undefined} No retorna dades
    */
    netejaFiltres: function(){
      cerca = angular.copy(cercaModel);
    },

    /**
    * @ngdoc method
    * @name app.service:Cerca#clearResultats
    * @methodOf app.service:Cerca
    * @param {object} parametres Objecte que conté un camp d'ordenació i un ordre
    * @description
    * Ordenem l'array de resultats en funció dels paràmetres passats
    * @return {undefined} No retorna dades
    */
    sortResultats: function(parametres){
      if(cerca.habitatges.length > 1){
        cerca.habitatges.sort($rootScope.ordenarArraysObj(parametres));
      }
    },
  };
});